/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: EmptyGroup.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef EMPTYGROUP_H
#define EMPTYGROUP_H

#include "SingleGroup.h"
#include <stdlib.h>
#include "StandardErrors.h"
#include "BaseGroupClass.h"

/**  This is a derived class for action groups that contain no
 *  actions.  This is mainly used for the instantaneous (zero
 *  time) action group which must be defined on all models.
 *
 *  @author Jay M. Doyle
 *  @version Mobius Version 1.0
 */
class EmptyGroup: public SingleGroup {

public:

	/**  Base constructor
	 */
	EmptyGroup();

	/** Base destructor
	 */
	~EmptyGroup();

	/** This method should never be called on this type
	 * of group. Calling the method does nothing.
	 * @see BaseActionClass
	 */
	void SetAction(BaseActionClass*);

	/** This method alway returns false since it doesn't
	 * have a clear definition if there are no members
	 * @see BaseActionClass
	 */
	bool ReactivationPredicate();

	/** This method alway returns false since it doesn't
	 * have a clear definition if there are no members
	 * @see BaseActionClass
	 */
	bool ReactivationFunction();

	/** This method should never be called since there
	 * are no action to select.  Calling this method
	 * will result in an inconsistant state error
	 * message.
	 */
	bool SelectAction();

	/** This method returns zero since there is no action
	 * from which to sample a distribution
	 * @see BaseActionClass
	 */
	double SampleDistribution();

	/** This method does nothing and returns a NULL pointer
	 * @see BaseActionClass
	 */
	BaseActionClass* Fire();

	/** This method always returns zero since the
	 * group has no memebers
	 */
	double Probability(BaseActionClass*);

	/** This method always returns false by default
	 */
	bool EnablingChange(void);

	/** This method always returns false by default
	 */
	bool Enabled();

	/** This method always returns 0 by default
	 */
	double Weight();

	/** This method always returns 0 by default
	 */
	double Rate();

	/** This method does nothing
	 */
	void SetImpulseList(int, double);

	/** This method does nothing
	 */
	void CreateImpulseList(int);

	/** This method always returns 0 by default
	 */
	int Rank();

}; // end EmptyGroup class

#endif // end EMPTYGROUP_H
