/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef _FAST_SET_H_
#define _FAST_SET_H_

#include <set>
#include <assert.h>

class FastSet {
public:
	static int number1;
	static int number2;
	FastSet();
	~FastSet();

	//return true if it was actually inserted
	bool insert(const int& item);
	//return true if found
	bool find(const int& item) const;
	void clear();
	bool empty() const;
	int& operator[](int index);
	int size() const;
	void NoMoreModifications();

private:
	bool is_vec_rep;
	union {
		struct {
			int* vec;
			int size;
		} vec_rep;
		std::set<int> * set_rep;
	} rep;
};

/*
 template<class T>
 class FastSet {
 public:
 static int number1;
 static int number2;
 FastSet() {
 rep.set_rep = new set<T>;
 is_vec_rep = false;
 number1++;
 printf("c(%d)\n", number1);
 }

 ~FastSet() {
 if (is_vec_rep)
 delete[] rep.vec_rep.vec;
 else
 delete rep.set_rep;
 }

 //return true if it was actually inserted
 bool insert(const T& item) {
 assert(!is_vec_rep);
 return rep.set_rep->insert(item).second;
 }

 //return true if found
 bool find(const T& item) const {
 assert(!is_vec_rep);
 return rep.set_rep->find(item) != rep.set_rep->end();
 }

 bool empty() const {
 assert(!is_vec_rep);
 return rep.set_rep->empty();
 }

 T& operator[](int index) {
 assert(is_vec_rep);
 return rep.vec_rep.vec[index];
 }

 int size() const {
 assert(is_vec_rep);
 return rep.vec_rep.size;
 }

 void NoMoreModifications() {
 assert(!is_vec_rep);
 //Copy the set into a newly created vector
 int size = rep.set_rep->size();
 T* vec = NULL;
 if (size > 0) {
 vec = new T[size];
 set<T>::iterator it;
 int i = 0;
 for (it = rep.set_rep->begin(); it != rep.set_rep->end(); it++, i++)
 vec[i] = *it;
 }
 //remove the set from the memory and replace it with the vector
 //representation
 delete rep.set_rep;
 number2++;
 printf("v(%d)\n", number2);
 is_vec_rep = true;
 rep.vec_rep.vec = vec;
 rep.vec_rep.size = size;
 }

 private:
 bool is_vec_rep;
 union {
 struct {
 T* vec;
 int size;
 } vec_rep;
 set<T>* set_rep;
 } rep;
 };

 template<class T>
 int FastSet<T>::number1 = 0;

 template<class T>
 int FastSet<T>::number2 = 0;
 */

#endif
