/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File:  Place.h
 *
 *      Authors:  Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1999 The Board of Trustees of the University of Illinois
 *                         All rights reserved.
 *
 \*****************************************************************************/

#ifndef BUCKET_H_
#define BUCKET_H_

#include "../state/SharableSV.h"
#include "../StandardErrors.h"
#include "../debug.h"
#include <iostream>

/** This class describes the base state variable class for stochastic activity networks
 *
 *   @author Jay M. Doyle
 *   @version Mobius version 1.0
 */
class Bucket: public SharableSV<short> {

public:

	/**
	 * Default constructor for Bucket
	 */
	Bucket();

	/**
	 * Constructor that takes the bucket's name
	 *
	 *  @param TheBucketName The name of the bucket at initialization time
	 */
	explicit Bucket(char* TheBucketName);

	/**
	 *  This contructor takes both the bucket's initial name and marking.
	 *
	 *  @param TheBucketName The initial name of the bucket
	 *  @param TheInitialMarking The initial bucket marking
	 */
	Bucket(char* TheBucketName, short TheInitialMarking);

	/**
	 * This method returns the value of the SAN bucket as an Lvalue
	 *
	 * @return The value of the mark (as an Lvalue)
	 */
	inline short& Mark() {
		return *TheMark;
	}

	/** This method returns the bucket's marking (by value)
	 */
	short getMark();

	/** This method sets the value of the bucket's mark
	 *
	 *  @param TheNewMarking The new value of the SAN bucket
	 */
	void setMark(short TheNewMarking);

	/**
	 *  A method used for debugging
	 */
	void test();

protected:

	/**
	 * The protected data structure that holds the bucket's marking
	 */
	short* TheMark;

}; // end class Bucket

#endif

