/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File:  ReadOnlyPlace.h
 *
 *      Authors:  Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1999 The Board of Trustees of the University of Illinois
 *                         All rights reserved.
 *
 \*****************************************************************************/

#ifndef READONLYBUCKET_H_
#define READONLYBUCKET_H_

#include "../state/ReadOnlySV.h"
#include "../StandardErrors.h"
#include <iostream>

/**
 *  This is the default implementation of
 *  the Mark method if no function is used
 *  to specify the value of the functionally
 *  shared Bucket.  It prints out an error message
 *  to standard error.
 */
short defaultMessage();

/**
 *   This class describes a Buckets whose state
 *   is defined in terms of a function of some
 *   other state in the model (possibly in an another
 *   atomic model).
 *
 *   @author Jay M. Doyle
 *   @version Mobius version 1.0
 */
class ReadOnlyBucket: public ReadOnlySV<short> {

public:

	/**
	 * Default constructor for ReadOnlyBucket
	 */
	ReadOnlyBucket();

	/**
	 * Constructor that takes the place's name
	 *
	 *  @param TheReadOnlyBucketName The name of the place at initialization time
	 */
	explicit ReadOnlyBucket(char* TheBucketName);

	/**
	 *  Destructor
	 */
	virtual ~ReadOnlyBucket();

	/**
	 * This method returns the value of the SAN place as an Lvalue
	 *
	 * @return The value of the mark (as an Lvalue)
	 */
	short Mark();

	/**
	 *  This method returns the place's marking (by value)
	 */
	short getMark();

	/** This method sets the value of the place's mark
	 *
	 *  @param TheNewMarking The new value of the SAN place
	 */
	void setMark(short TheNewMarking);

	/**
	 *  This method should never be called.
	 *  It's default implementation prints
	 *  an error message.
	 */
	void SetState(void* NewState);

	/**
	 *  This method copies the value of the
	 *  state variable in the current state
	 *  to the specified memory location.
	 *
	 *  @param MemoryLocation The location in
	 *   memory where the state value should be
	 *   stored
	 */
	void CurrentState(void* MemoryLocation);

	/**
	 *  Function used for debugging
	 */
	void test();

protected:

	/**
	 *  This is a pointer to a function that
	 *  defines the value of the ReadOnlyBucket.
	 *  This function should be defined by the
	 *  composed model.
	 */
	short (*valueFunction)();

}; // end class ReadOnlyBucket

#endif

