/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: MobiusEnums.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef MOBIUS_ENUMS_H_
#define MOBIUS_ENUMS_H_

/** These specific enumerations For mobius base classes
 * @author Jay M. Doyle
 * @version Mobius Version 1.0
 */

/** This enumeration describes the different types of probability distributions action
 * can have in the Mobius framework.
 */
enum Distribution {
	Proprietary,
	Exponential,
	Deterministic,
	Geometric,
	Weibull,
	Normal,
	Lognormal,
	Erlang,
	Triangular,
	Gamma,
	Beta,
	Uniform,
	Binomial,
	NegativeBinomial,
	HyperExponential,
	ConditionalWeibull,
	HypoExponential,
	Pareto,
	Instantaneous
};

/**  This type desribes all the different execution policies an action
 can have.
 */
enum ExecutionPolicyType {
	RaceResampling, RaceEnabled, RaceAge
};


/** This is type used to describe the contents of an array
 */
enum ArrayType {
	ArrayType_SHORT,
	ArrayType_INT,
	ArrayType_CHAR,
	ArrayType_DOUBLE,
	ArrayType_FLOAT,
	ArrayType_ARRAY,
	ArrayType_STRUCT,
	ArrayType_UNORDERED,
	ArrayType_EMPTY
};

/**
 *  This type is used to save the accumulation state of
 *  the performance variable worker during simulation. <BR>
 *
 *  <UL>
 *  <LI> acc_later -- the simulations has not reach the
 *                    start time of the interval
 *  <LI> acc_now   -- the simulation is currently inside
 *                    the accumulation interval
 *  <LI> acc_done  -- the simulation is past the accumulation
 *                    interval
 *  </UL>
 *
 */
enum AccumulationState {
	acc_later, acc_now, acc_done, acc_ignore
};

/**
 *  This type is used to describe the different
 *  type of performance variables available in
 *  mobius.
 *
 *  <UL>
 *  <LI> terminating -- the performance variable value
 *       is calculated at some time <I>t</I>
 *  <LI> steady_state -- the performance variable value
 *       is calculated as <I>t</I> approaches infinite.
 *  </UL>
 */
enum PVType {
	terminating, steady_state
};

/**
 *  This type defines the possible output file formats
 *  for the state space generator.  I know that there
 *  are supposed to be more than two that why I created
 *  an enumerated type.
 */
enum OutputFileType {
	ASCII, Binary
};

/**
 *  the types of reward variables, this is more specific than
 *  terminating verses steady_state.  eventually, RewardType and
 *  PVType should be unified, likely replacing the usage of PVType
 *  with appropriate uses of RewardType.  I am adding RewardType now,
 *  and don't want to take the risk of removing and unifying it with
 *  PVType at the same time.  This enumeration cooresponds with the
 *  enumeration in the Java side.
 */
enum RewardType {
	INSTANT_OF_TIME = 0, INTERVAL_OF_TIME, TIMEAVE_INTERVAL_OF_TIME, STEADY_STATE, UNDEFINED
};
#endif // MOBIUS_ENUMS_H_
