/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: MobiusThread
 *
 *      Authors: Tod Courtney
 *      Creation Date: 2003
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef _MOBIUS_THREAD_H_
#define _MOBIUS_THREAD_H_

#include <cstdlib>

#if MINGW || _WIN32
#include <process.h>
#include <windows.h>
#define THREAD HANDLE
#define THREADRETURN DWORD
typedef LPTHREAD_START_ROUTINE THREADFUNC;
#else
#include <pthread.h>
#include <unistd.h>
#define THREAD pthread_t
#define THREADRETURN void *
typedef THREADRETURN( *THREADFUNC)(void*);
#endif

class MobiusThread {
protected:
	THREADFUNC _threadFunc;
#if MINGW  || _WIN32
	LPDWORD _threadID;
#endif
	void * _threadData;
	THREAD _threadHandle;
public:
	MobiusThread(THREADFUNC func, void* data);

	void Start();
	void WaitUntilDone();
	static void Sleep(int msec);
};

class MobiusCond;

class MobiusMutex {
	friend class MobiusCond;
protected:
	char _name[64];
	static int _globalID;
#if MINGW  || _WIN32
	HANDLE _mutex;
#else
	pthread_mutex_t _mutex;
#endif

public:
	MobiusMutex();
	~MobiusMutex();
	void Get();
	void Release();
};

class MobiusCond {
protected:
#if MINGW || _WIN32
	int _waiters_count;
	// Number of waiting threads.

	CRITICAL_SECTION _waiters_count_lock;
	// Serialize access to <waiters_count_>.

	HANDLE _sema;
	// Semaphore used to queue up threads waiting for the condition to
	// become signaled.

	HANDLE _waiters_done;
	// An auto-reset event used by the broadcast/signal thread to wait
	// for all the waiting thread(s) to wake up and be released from the
	// semaphore.

	size_t _was_broadcast;
	// Keeps track of whether we were broadcasting or signaling.  This
	// allows us to optimize the code if we're just signaling.
#else
	pthread_cond_t _cond;
#endif
public:
	MobiusCond();
	~MobiusCond();
	int Wait(MobiusMutex& external_mutex);
	int Signal();
};

#endif
