/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: PEPAActivity.h
 *
 *      Authors: Graham Clark, Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 2000
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef PEPAACTIVITY_H_
#define PEPAACTIVITY_H_

#include "../BaseGroupClass.h"
#include "../debug.h"
#include "PEPAModel.h"

class PEPAModel;

/**
 *  This class describes the operation of PEPA Activities -- the
 *  formalism specific action for PEPA models
 *
 *  @author Graham Clark
 *  @version Mobius Version 1.0 Development
 */
class PEPAActivity: public BaseGroupClass {

public:

	bool NewEnabled;
	bool OldEnabled;

	PEPAActivity();
	~PEPAActivity();

	PEPAActivity(char* TheActionName, int TheGroupIDNumber, Distribution TheDistributionType, ExecutionPolicyType TheExecutionPolicy,
			int TheNumberOfAffectedStateVariables, int TheNumberOfEnablingStateVariables, bool TheReactivation);

	void ActivityInitialize(char* TheActionName, int TheGroupIDNumber, Distribution TheDistributionType, ExecutionPolicyType TheExecutionPolicy,
			int TheNumberOfAffectedStateVariables, int TheNumberOfEnablingStateVariables, bool TheReactivation);

	virtual void LinkVariables()=0;

	//virtual BaseActionClass* Fire()=0;

	bool SelectAction();

	void CalculateWeightDistribution();

	bool EnablingChange();

	double Probability(BaseActionClass* TheAction);

	bool IsAnAffectingStateVariable(BaseStateVariableClass* TheStateVariable);

	inline void setGroupId(int TheNewID) {
		GroupID = TheNewID;
	}

	bool Enabled();
	double Weight();
	bool ReactivationPredicate();
	bool ReactivationFunction();
	double* ReturnDistributionParameters();
	int Rank();
	virtual BaseActionClass* Fire()=0;

	virtual bool GuardSatisfied()=0;

	virtual bool OutputActivitiesDontClash()=0;

	void RegisterModel(PEPAModel *model);
	void SetNumberOfComponents(int num);
	void RegisterCompList(short **list);
	void RegisterSuccessorState(int seqproc, short substate);
	void RegisterEnabledInState(int seqproc, short substate);
	bool EnabledByCurrentTerms();
	void UpdateCurrentTerms();

protected:

	double min(double a, double b);

	double* TheDistributionParameters;

	int NumAssociatedComps;

	// gcla - the entire list of program counters
	short **CompList;

	// a list of which of the above are affected by this activity
	short *FeaturedCompList;

	short *FeaturedSuccStateList;

	short *FeaturedEnabledStateList;

	PEPAModel *theModel;

private:

	int CurrentSuccessorCounter;
	int CurrentEnabledCounter;

};

#endif
