/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: Place.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef PLACE_H_
#define PLACE_H_

#include "../state/SharableSV.h"

/** This class describes the base state variable class for stochastic activity networks
 *
 *   @author Jay M. Doyle
 *   @version Mobius version 1.0
 */
class Place: public SharableSV<short> {

public:

	/**
	 * Default constructor for Place
	 */
	Place();

	/**
	 * Constructor that takes the place's name
	 *
	 *  @param ThePlaceName The name of the place at initialization time
	 */
	explicit Place(char* ThePlaceName);

	/**
	 *  This contructor takes both the place's initial name and marking.
	 *
	 *  @param ThePlaceName The initial name of the place
	 *  @param TheInitialMarking The initial place marking
	 */
	Place(char* ThePlaceName, short TheInitialMarking);

	/**
	 * This method returns the value of the SAN place as an Lvalue
	 *
	 * @return The value of the mark (as an Lvalue)
	 */
	inline short& Mark() {
		return *TheMark;
	}

	/** This method returns the place's marking (by value)
	 */
	short getMark();

	/** This method sets the value of the place's mark
	 *
	 *  @param TheNewMarking The new value of the SAN place
	 */
	void setMark(short TheNewMarking);

	/**
	 *  A method used for debugging
	 */
	void test();

protected:

	/**
	 * The protected data structure that holds the place's marking
	 */
	short* TheMark;

}; // end class Place

#endif

