/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: StandardErrors.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef STANDARD_ERRORS_H_
#define STANDARD_ERRORS_H_

//#include "../gen/Defines4SymSSpGen.h"
#include <stdlib.h>
#include <iostream>

/** This is a list of standardly used error messages
 * Their definitions can be found in StandardErrors.cpp
 *
 *  @author Jay M. Doyle
 *  @version Mobius Version 1.0
 */

/** Prints an error message to cerr if an user tries to allocate space that is no
 * longer avaiable
 */
void MemoryAllocationError();

/** Prints out a generic memory error message to cerr
 */
void MemoryError();
void ErrorEmptyParameter(const char *);

/** Prints out an error message alerting the user to an inconsistant state
 */
void InconsistantState();

/** Prints out an error message alerting the user to an error he/she made
 */
void UserError();

/** Prints out an error message is message if equivalence starting is tried on two
 * dissimilar array types
 *
 * @param FirstStateVariableType The first state variable type
 * @param SecondStateVariableType The second state variable type
 */
void ArrayTypeMismatchError(char* FirstStateVariableType, char* SecondStateVariableType);

/** Prints out an error message is message if equivalence starting is tried on two
 * dissimilar array types
 *
 * @param FirstStateVariableType The first state variable type
 * @param SecondStateVariableType The second state variable type
 */

void StructTypeMismatchError(char* FirstStateVariableType, char* SecondStateVariableType);

/** Prints out invalid array type error message to cerr
 */
void InvalidArrayType();

/** Prints out invalid struct type error message to cerr
 */
void InvalidStructType();

/** Prints out invalid unordered typ error message to cerr
 */
void InvalidUnorderedType();

/**
 * Prints an array out of bounds error message
 *
 *   @param TheLocation The location of array out of
 *          bounds error
 */
void ArrayOutOfBoundsError(char* TheLocation);

/** Prints out a generic sync error message to cerr
 */

inline void MaxZeroTransFired(unsigned long maxnum) {
	std::cerr << "Error:  Maximum number of zero time transition" << "recursive firings exceeded" << "(" << maxnum << ")\nPossible undetermined model" << std::endl
			<< "Please check the model and try again" << std::endl;
}
; // MaxZeroTransFired(){

inline void WrongDistributionType() {
	std::cerr << "You can only explore state space for models";
	std::cerr << "containing only exponential and optional instantenous";
	std::cerr << "activities" << std::endl;
}
;// WrongDistributionType

/*void MaxZeroTransFired();
 void WrongDistributionType(); //markov and optional
 //immediated activities are acceptable */

class SyncError {
public:
	void WrongDistributionType();//either markov or immediated are acceptable
	void NoAction();//no Action found in model
};

#endif
