/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef TRACE_FILE
#define TRACE_FILE

#include <iostream>
#include <fstream>
#include <memory>

// these two externs are added for backwards compatibility with sgen code.
extern std::ostream *trout;
extern int TraceLevel;

// the numbers 1-4 should correspond to the levels used in the java code
#define TRACEOFF 0
#define TRACEA 1
#define TRACEB 2
#define TRACEC 3
#define TRACEZ 4

class BaseModelClass;
class BaseActionClass;
class BaseStudyClass;
class PVModel;

namespace tf
{
    inline std::ostream& newl(std::ostream& cout)
    {
        return cout << "\n";
    }
}

class TraceFile {
public:
	TraceFile();
	TraceFile(const char* fname);
	TraceFile(std::ostream * stream);

	// access methods, printf, etc
	virtual void printHeader(PVModel* ThePVModel);
	virtual void printFooter();
	virtual void printInitialState(BaseModelClass* TheModel, BaseStudyClass* theStudy);
	virtual void printState(BaseModelClass* TheModel);
	virtual void printState(PVModel* PVmodel);
	virtual void printFireAction(BaseActionClass* CurrAction, double CurrentTime);
	virtual void printPreFireAction(BaseActionClass* CurrAction);
	virtual void printPostFireAction(BaseActionClass* CurrAction, BaseActionClass* TempAction);
	virtual void newBatch();
	void setTraceLevel(int TheLevel);
	int getTraceLevel();

	// Try to use this as minimally as possible
	virtual int printf(int level, const char* format, ...);
	virtual void printStr(int levek, char* str);

	virtual ~TraceFile() = default;

protected:
	int mTraceLevel;
	std::unique_ptr<std::ostream> traceStream;
	std::ostream* tracefileout;
};

#endif TRACE_FILE
