/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: TraceFileContainer
 *
 *      Authors: Michael McQuinn
 *      Creation Date: 2005
 *
 *      Copyright (c) 1994-2005 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef TRACE_FILE_CONTAINER
#define TRACE_FILE_CONTAINER

#include "TraceFile.h"
#include <vector>

class BaseStudyClass;

class TraceFileContainer: public TraceFile {
public:
	TraceFileContainer();

	void AddTraceFile(TraceFile* trace);
	void RemoveTraceFile(TraceFile* trace);

	// Other accesses, printf, etc
	void printHeader();
	void printHeader(PVModel* ThePVModel) override;
	void printFooter() override;
	void printState();
	void printState(BaseModelClass *TheModel) override;
	void printState(PVModel *PVmodel) override;
	void printInitialState();
	void newBatch() override;
	void printFireAction(BaseActionClass *CurrAction, double CurrentTime) override;
	void printPreFireAction(BaseActionClass *CurrAction) override;
	void printPostFireAction(BaseActionClass* CurrAction, BaseActionClass *TempAction) override;

	int printf(int level, const char* format, ...) override;
	void printStr(int level, char * str) override;

	void addStudy(BaseStudyClass* theStudy);
	void addPVModel(PVModel* ThePVModel);

private:
	BaseStudyClass* mpBaseStudy;
	PVModel* mpPvModel;
	// data structures here
	std::vector<TraceFile*> traceFileVec;
};

#endif TRACE_FILE_CONTAINER
