/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef TRACE_FILE_MOBIUS
#define TRACE_FILE_MOBIUS

#include "TraceFile.h"

#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdarg>
#include <cstring>

class TraceFileMobius: public TraceFile {
public:
	TraceFileMobius();
	TraceFileMobius(const char* fname);
	TraceFileMobius(std::ostream * stream);

	// access methods, printf, etc
	void printHeader(PVModel* ThePVModel) override;
	void printFooter() override;
	void printState();
	void printInitialState(BaseModelClass* TheModel, BaseStudyClass* theStudy) override;
	void printState(BaseModelClass* TheModel) override;
	void printState(PVModel* PVmodel) override;
	void printFireAction(BaseActionClass* CurrAction, double CurrentTime) override;
	void printPreFireAction(BaseActionClass* CurrAction) override;
	void printPostFireAction(BaseActionClass* CurrAction, BaseActionClass* TempAction) override;
	void newBatch() override;

};

/**
 *  this method is added here right before Mobius 2.0 was released, to
 *  support the code in sgen, which previously used TraceOut.  We need
 *  to remove TraceOut from the source, and its more straightfowrard
 *  at this phase to add this method here than rewrite all the methods
 *  that use TraceOut.
 */
inline int Traceprintf(int level, char *format, ...) {
	static char str[1000];
	static char buffer[1000];
	static va_list ap;
	if (TraceLevel >= level) {
		char* new_format;
		/*
		 To print long long ints using printf family of functions, we can use the %lld
		 specifier in *nix OSes and also when using cygwin's C/C++ compiler without
		 the -mno-cygwin option . However, since the -mno-cygwin causes the cygwin
		 compiler to use Windows native libraries and the latter do not support printing
		 long long ints in a standard way, the %lld option do not work. Instead we need to
		 use %I64d specifier. The following code replaces all occurrences of %lld
		 in parameter 'format' by %I64d if MINGW (-mno-cygwin) is defined.
		 */
#ifdef MINGW
		/*
		 The following code constructs new_format such that it is the
		 same as format except that all occurences of %lld are replaced
		 by %I64d. The only reason I'm using this rather complicated code
		 is that I would like to do the search and replace in one pass
		 , i.e., in O(n) time where n is the length of format parameter (derisavi)
		 */

		new_format = buffer;
		new_format[0] = '\0';
		char* rest = format;
		char* instance;
		const int len_of_orig_str = strlen("%lld");
		const int len_of_rep_str = strlen("%I64d");
		while (instance = strstr(rest, "%lld")) {
			strncpy(new_format, rest, instance - rest);
			strncpy(new_format + (instance - rest), "%I64d", len_of_rep_str+1);
			new_format += instance - rest + len_of_rep_str;
			rest = instance + len_of_orig_str;
		}
		strcpy(new_format, rest);
		format = buffer;
#endif
		va_start(ap, format);
		vsprintf(str, format, ap);
		(*trout) << str;
		trout->flush();
		if ((level == TRACEOFF) && (trout != &std::cout)) {
			std::cout << str;
			std::cout.flush();
		}
		va_end(ap);
	}

	return 0;
}

#endif TRACE_FILE_MOBIUS
