/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef TRACE_FILE_XML
#define TRACE_FILE_XML

#include "TraceFile.h"

#include <vector>
#include <map>

#include "../Performance_Variables/PVModel.hpp"

class TraceFileXML: public TraceFile {

public:
	TraceFileXML();
	TraceFileXML(const char* fname);
	TraceFileXML(std::ostream * stream);
	virtual ~TraceFileXML();

	// access methods, printf, etc
	void printHeader(PVModel* ThePVModel) override;
	void printFooter() override;
	void printState();
	void printState(BaseModelClass* TheModel) override;
	void printInitialState(BaseModelClass* TheModel, BaseStudyClass* theStudy) override;
	void printState(PVModel* PVmodel) override;
	void printFireAction(BaseActionClass* CurrAction, double CurrentTime) override;
	void printPreFireAction(BaseActionClass* CurrAction) override;
	void printPostFireAction(BaseActionClass* CurrAction, BaseActionClass* TempAction) override;
	void newBatch() override;

	void printHeader(BaseModelClass* TheModel, int ParentId);
	void printHeaderInteractions(BaseModelClass* TheModel);

private:
	//std::ofstream tracefileout;
	std::vector<std::string> stateNames;
	std::vector<std::string> UsedStateNames;
	std::vector<std::string> UsedActionNames;
	std::vector<std::vector<int> > UsedStateNamesIds; // added 01/28/08 by Ruth Lamprecht

	bool addState(const char* stateName);
	bool addUsedStateName(const char* usedName, int ParentId, int NameId);
	bool inUsedStateName(const char* usedName);
	int getModelMap(const char* name, int ModelId);
	int runNum;
	double LastActionTime;

	struct comp {
		bool operator()(const BaseModelClass* m1, const BaseModelClass* m2) const {
			// return m1 < m2;
			return true;
		}
	};

	typedef std::pair<BaseModelClass*, std::vector<BaseStateVariableClass*> *> entry;

	std::map<BaseModelClass*, std::vector<BaseStateVariableClass*> *, comp> ModelMap;
};

#endif TRACE_FILE_XML
