/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: Utility.hpp
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef UTILITY_HPP_
#define UTILITY_HPP_

#include "MobiusTypeDefs.h"
#include "debug.h"
#include <iostream>
#include <cstring>

/**
 *  This method allows the user to access
 *  data elements across from memory, even
 *  across word boundaries.
 *
 *  @param TheLocation The location in memory
 *    where the data resides
 *  @return The value of the data at TheLocation
 */
template<class C> C readMemory(void* TheLocation) {
	C ReturnValue;
	std::memcpy(&ReturnValue, TheLocation, sizeof(C));
	return ReturnValue;
} // end template <class C> C readMemory(void*)

/**
 *
 */
template<class C> void readMemory2(void* TheLocation, C* TheValue) {
	DS3("Reading value from", TheLocation);
	DS3("Saving value to location", TheValue);
	std::memcpy(TheValue, TheLocation, sizeof(C));
} // end readMemory2(void*, C*)

/**
 *  This method allows the user to access
 *  an array of data elements across from memory, even
 *  across word boundaries.
 *
 *  @param TheLocation The location in memory
 *    where the data resides
 *  @param TheValue The location where this data should be stored.
 *  @param TheSize The size of the array to be read in
 *
 */
template<class C> void readArray(C* TheValue, void* TheLocation, int TheSize) {
	std::memcpy(TheValue, TheLocation, sizeof(C) * TheSize);
} // end template <class C> C readMemory(void*)

/**
 *  This method allows the user to write
 *  a value into memory, even across word
 *  boundaries.
 *
 *  @param TheValue The value to be written
 *    to memory.
 *  @param TheLocation The location where
 *    the memory should be stored.
 */
template<class C> void writeMemory(C TheValue, void* TheLocation) {
	DS3("Writing value to location", TheLocation);
	std::memcpy(TheLocation, &TheValue, sizeof(C));
} // end template <class C> void writeMemory(C, void*)

/**
 *  This method allows the user to write
 *  an array into memory, even across word
 *  boundaries.
 *
 *  @param TheValue The value to be written
 *    to memory.
 *  @param TheLocation The location where
 *    the memory should be stored.
 *  @param The number of elements in the array
 *    to be written to memory
 */
template<class C> void writeArray(C* TheValue, void* TheLocation, int TheSize) {
	std::memcpy(TheLocation, TheValue, sizeof(C) * TheSize);
} // end template <class C> void writeMemory(C, void*)

#endif
