/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2011 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/**
 * @file Access.h
 * This file contains the Access class which is a Mobius state variable.
 */

#ifndef ACCESS_H_
#define ACCESS_H_

#include "ADVISEStateVariable.h"

/**
 * The Access class represents the Access state variable.
 */
class Access: public ADVISEStateVariable<short> {
public:
	/**
	 * Constructor
	 */
	Access();

	/**
	 * Constructor
	 * @param[in] theName The name of the access
	 */
	explicit Access(char *theName);

	/**
	 * Constructor
	 * @param[in] theName The name of the access
	 * @param[in] initialValue The initial value of the access
	 */
	Access(char *theName, short initialValue);

	Access(const Access&) = delete;
	Access& operator=(const Access&) = delete;

	/**
	 * Destructor
	 */
	virtual ~Access();
};

#endif /* ACCESS_H_ */
