/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2011 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/**
 * @file Goal.h
 * This file contains the Goal class which is a Mobius state variable.
 */
#ifndef GOAL_H_
#define GOAL_H_

#include "ADVISEStateVariable.h"

/**
 * The Goal class represents the Goal state variable
 */
class Goal : public ADVISEStateVariable<short> {
public:
	/**
	 * Constructor
	 */
	Goal();
	/**
	 * Constructor
	 * @param[in] theName The goal name
	 */
	explicit Goal(char *theName);
	/**
	 * Constructor
	 * @param[in] theName The goal name
	 * @param[in] initialValue The goal's initial value
	 */
	Goal(char *theName, short initialValue);
	/**
	 * This function gets the goal's payoff value
	 */
	inline double getPayoff() { return payoff; }
	/**
	 * This function sets the goal's payoff value
	 * @param[in] val The new payoff value
	 */
	inline void setPayoff(double val) { payoff = val; }
	/**
	 * Destructor
	 */
	virtual ~Goal();

	Goal(const Goal&) = delete;
	Goal& operator=(const Goal& rhs) = delete;

protected:
	/**
	 * The goal's payoff value
	 */
	double payoff;
};

#endif /* GOAL_H_ */
