/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2011 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/**
 * @file StepWeight.h
 * This file contains the StepWeight class which is a Mobius state variable.
 */
#ifndef STEPWEIGHT_H_
#define STEPWEIGHT_H_

#include "ADVISEStateVariable.h"
/**
 * The StepWeight class is a state variable used to determine the case weight of an attack step for the adversary decision algorithm. Since the current implementation of the adversary decision is deterministic, all step weight values are 0 except the one that was deterministically chosen. If we introduce stochasticity in the future, this is what we'll use.
 */
class StepWeight : public ADVISEStateVariable<short> {
public:
	/**
	 * Constructor
	 */
	StepWeight();

	/**
	 * Constructor
	 * @param[in] theName The name of the step weight
	 */
	explicit StepWeight(char *theName);

	/**
	 * Constructor
	 * @param[in] theName The name of the step weight
	 * @param[in] initialValue The initial value of the step weight.
	 */
	StepWeight(char *theName, short initialValue);

	StepWeight(const StepWeight&) = delete;
	StepWeight& operator=(const StepWeight&) = delete;

	/**
	 * Destructor
	 */
	virtual ~StepWeight();

};

#endif /* STEPWEIGHT_H_ */
