/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2011 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/**
 * @file SystemSV.h
 * This file contains the SystemSV class.
 */
#ifndef SYSTEMSV_H_
#define SYSTEMSV_H_

#include "ADVISEStateVariable.h"
/**
 * The SystemSV class is the parent class for all system state variables.
 */
template <class C>
class SystemSV: public ADVISEStateVariable<C> {
public:
	/**
	 * Constructor
	 */
	SystemSV() : ADVISEStateVariable<C>() {
	}

	/**
	 * Constructor
	 * @param[in] theName The name of the system state variable
	 */
	explicit SystemSV(char *theName) : ADVISEStateVariable<C>(theName) {
	}

	/**
	 * Constructor
	 * @param[in] theName The name of the system state variable
	 * @param[in] initialValue The initial value of the system state variable
	 */
	SystemSV(char *theName, const C & initialValue) : ADVISEStateVariable<C>(theName, initialValue) {
	}

	SystemSV(const SystemSV<C>&) = delete;
	SystemSV<C>& operator=(const SystemSV<C>&) = delete;

	/**
	 * Destructor
	 */
	virtual ~SystemSV() {
	}
};

#endif /* SYSTEMSV_H_ */
