/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: ReadOnlySV.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __READONLYSTATEVARIABLE_H_
#define __READONLYSTATEVARIABLE_H_

#include <iostream>
#include <cassert>

#include "../BaseStateVariableClass.h"

/**  This is the base class for all read-only, functionally
 *  shared state variables
 */
template<class C> class ReadOnlySV: public BaseStateVariableClass {

public:

	/** Base costructor
	 */
	ReadOnlySV();

	/** Overloaded constructor that takes the state variable name
	 *
	 * @param  TheName The name of the read only state variable
	 */
	ReadOnlySV(char* TheName);

	/**
	 * Destructor
	 */
	virtual ~ReadOnlySV();

	/**
	 *  This method is overloaded to return 0
	 *  since no state is stored in read-only
	 *  state variables
	 */
	inline int StateSize() {
		return 0;
	}

	/**
	 * This method prints an error message since
	 * the state cannnot be set on
	 * a functionally shared state variable
	 */
	void SetState(void*);

	/**
	 *  This method assigns the function that defines
	 *  the state variable value.
	 *
	 *  @param TheStateFunction The function that should
	 *    be used to define the value of the state variable
	 */
	void setFunction(C(*TheStateFunction)());

	/**
	 * Fullfills requirement for BaseStateVariable
	 */
	void CurrentState(void*);

	/**
	 * Fullfills requirement for BaseStateVariable
	 */
	void printState();

	void share(BaseStateVariableClass* other);
protected:

	/**
	 * Pointer to fuction that assigns state
	 */
	C (*getValue)();

private:

}; // end ReadOnlyStateVariable

#endif
