/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: SharedPointer.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __SHARED_POINTER_H_
#define __SHARED_POINTER_H_

#include "../debug.h"
#include "../BaseStateVariableClass.h"
#include <iostream>
#include <stdio.h>

/** This class is an element of a linked list used to keep
 * track of all the shared state variable pointers
 */
template<class C> class SharedPointer {

public:

	/**  BaseConstructor
	 */
	SharedPointer() {
		CF("template <class C> SharedPointer<C>::SharedPointer()");
		TheSharedPointer = NULL;
		Next = NULL;
		Owner = NULL;
	} // end template <class C> SharedPointer<C>::SharedPointer()

	/**
	 *  Destructor
	 */
	~SharedPointer() {
		CF("template <class C> SharedPointer<C>::~SharedPointer()");
	} // end template <class C> SharedPointer<C>::SharedPointer()


	/** This method set the Neighbor value of the element
	 *
	 * @param TheNewNeighbor The element to which this element will
	 *        attach itself.
	 */
	void setNeighbor(SharedPointer* NewNeighbor) {
		Next = NewNeighbor;
	} // end template <class C> SharedPointer<C>::setNeighbor(SharedPointer<C>*)

	/**  This method returns this element's neighbor in the linked lis
	 *
	 * @return This element's neighbor in the linked list
	 */
	SharedPointer* getNeighbor() {
		return Next;
	}// end template <class C> SharedPointer<C>::getNeighbor()

	/** This method sets the value of the SharedPointer data member
	 *
	 * @param TheNewSharedPointer The new value for SharedPointer
	 */
	void setPointer(C** ThePointer) {
		TheSharedPointer = ThePointer;
	} // end template <class C> SharedPointer<C>::setPointer(C**)

	/** This method gets the value of the SharedPointer
	 *
	 * @return The value of the SharedPointer
	 */
	C** getPointer() {
		return TheSharedPointer;
	} // end template <class C> SharedPointer<C>::getPointer

	/** This method changes the location to which the shared state variable pointer points
	 * reflecting a migration of the shared state variable location.
	 *
	 */
	void changePointer(C* ThePointer) {
		*TheSharedPointer = ThePointer;
	} // end template <class C> SharedPointer<C>::changePointer(C*)

	/**
	 *  This method sets the value of the Owner
	 *  data member.
	 *
	 *  @param TheOwner The state variable that
	 *       has the state pointer located at
	 *       TheSharedPointer.
	 */
	void setOwner(BaseStateVariableClass* TheOwner) {
		CF("Entering template <class C> SharedPointer<C>::setOwner(BaseStateVariableClass*");
		Owner = TheOwner;
	} // end template <class C> SharedPointer<C>::setOwner(BaseStateVariableClass*)

	/**
	 *  This method returns a pointer to state variable
	 *  that contians TheSharedPointer.
	 *
	 *  @return The value of Owner
	 */
	BaseStateVariableClass* getOwner() {
		CF("Entering template <class C> SharedPointer<C>::getOwner()");
		return Owner;
	} // end template <class C> SharedPointer<C>::getOwner()


private:

	/**  The location of the pointer to the shared state variable
	 */
	C** TheSharedPointer;

	/**
	 *  This is the state variable that corresponds to
	 *  state poninter TheSharedPointer.
	 */
	BaseStateVariableClass* Owner;

	/**  A pointer to the next element in the linked list
	 */
	SharedPointer* Next;

}; // end SharedPointer

#endif
