/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: TestStructSV.h
 *
 *      Authors: Jay M. Doyle, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef TESTSTRUCTSV_H_
#define TESTSTRUCTSV_H_

#include "StructStateVariable.h"
#include "ShortStateVariable.h"
#include "IntStateVariable.h"
#include "../StandardErrors.h"
#include <iostream>

/**
 *  This class describes a test structured
 *  state variable used in testing sharing
 *  among parts of state variables.
 *
 *   @author Jay M. Doyle
 *   @version Mobius version 1.0
 */
class TestStructSV: protected StructStateVariable {

public:

	/**
	 * Default constructor for TestStructSV
	 */
	TestStructSV();

	/**
	 * Constructor that takes the sv's name
	 *
	 *  @param TheTestStructSVName The name of
	 *         the place at initialization time
	 */
	TestStructSV(char* TheTestStructSVName);

	/**
	 * This method returns the value of the SAN place as an Lvalue
	 *
	 * @return The value of the mark (as an Lvalue)
	 */
	inline short QualityOfService() {
		return getQualityOfService();
	}

	/**
	 *  This method returns the
	 *  place's marking (by value)
	 */
	inline short& servers() {
		return Servers.State();
	}

	/**
	 *  This method returns the
	 *  place's marking (by value)
	 */
	inline int& customers() {
		return Customers.State();
	}

	/**
	 *  This method returns the
	 *  place's marking (by value)
	 */
	inline short getQualityOfService() {
		if (Servers.getState() > Customers.getState())
			return 0;
		else if (Servers.getState() == Customers.getState())
			return 1;
		else
			return 2;
	} // getQualityOfService

	/**
	 * This method sets the value of the place's mark
	 *
	 *  @param TheNewServers The new number of servers
	 */
	inline void setServers(short TheNewServers) {
		Servers.setState(TheNewServers);
	}

	/**
	 * This method sets the value of the place's mark
	 *
	 *  @param TheNewCustomers The new number of customers
	 */
	void setCustomers(int TheNewCustomers) {
		Customers.setState(TheNewCustomers);
	}

	/**
	 *  Method used for testing
	 */
	void test();

	/**
	 *  A state variable that reflect the
	 *  number of servers operating
	 */
	ShortStateVariable Servers;

	/**
	 * A state variable that reflects the
	 * number of customers in the queue.
	 */
	IntStateVariable Customers;

protected:

	/**
	 *  This array describes the type of state variables
	 *  in this structured state variable.
	 */
	const ArrayType MyTypes[2] = { Short, Int };

}; // end class TestStructSV

#endif

