/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef DoubleFunction_H_
#define DoubleFunction_H_

#include <set>
#include <map>
#include <stdio.h>
#include <utility>

class TFunc {
public:
	TFunc() {
		f.Func = NULL;
	}
	~TFunc() {
		clear();
	}
	void clear() {
		if (f.Func == NULL)
			return;
		if (type == is_set) {
			f.Set->clear();
			delete f.Set;
		} else {
			for (std::map<int, TFunc*>::iterator it = f.Func->begin(); it != f.Func->end(); it++) {
				delete it->second;
			}
			f.Func->clear();
			delete f.Func;
		}
		f.Func = NULL;
	}

	union {
		std::map<int, TFunc*> * Func;
		std::set<int>* Set;
	} f;
	enum {
		is_func, is_set
	} type;

	bool isempty() {
		return f.Func == NULL;
	}
	int MaxDomain();
	void AddTuple(int* states, int states_size);
	void print();
};

class TMDMatrix {
public:
	TMDMatrix() {
		m.matrix = NULL;
	}
	~TMDMatrix() {
		clear();
	}
	void clear() {
		if (m.matrix == NULL)
			return;
		if (type == is_element) {
			delete m.element;
		} else {
			for (std::map<std::pair<int, int> , TMDMatrix*>::iterator it = m.matrix->begin(); it != m.matrix->end(); it++) {
				delete it->second;
			}
			m.matrix->clear();
			delete m.matrix;
		}
		m.matrix = NULL;
	}

	union {
		std::map<std::pair<int, int> , TMDMatrix*> * matrix;
		double* element;
	} m;
	enum {
		is_matrix, is_element
	} type;
	bool IsMatrix() {
		return type == is_matrix;
	}
	/*
	 typedef void (*recursion_functype)(int*, int*, double, int);
	 typedef void (*basecase_functype)(double);
	 */

	bool isempty() {
		return m.matrix == NULL;
	}
	int MaxDomain();
	void InsertTuple(const int* const sources, const int* const destinations, double element, int path_len);
	void AddTuple(const int* const sources, const int* const destinations, double element, int path_len);
	void AddMatrix(TMDMatrix& mat);

	void print();
};

#endif // DoubleFunction_H_
