/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __SSNODE_H_
#define __SSNODE_H_

#include "typedefs.h"
#include "mddnode.h"

#include <stdlib.h>
#include <iostream>
#include <assert.h>
#include <queue>
#include <stdio.h>
#include <set>

using namespace std;

//This type is used to represent the state in which a node of the
//MDD is when running the DFS algorithm on a part of the MDD to
//compute (useful) nodes between two specified nodes
typedef enum {
	undetermined, useless, useful, marked
} TNodeStatus;

#if (HAS_FORWARD_LIST)
	class SSNode;
	typedef std::forward_list<SSNode*> ssnode_list_t;
	typedef std::forward_list<std::pair<SSNode*, TNodeStatus> > ssnode_tnodestatus_list_t;
#else
	class SSNode;
	typedef slist<SSNode*> ssnode_list_t;
	typedef slist<pair<SSNode*, TNodeStatus> > ssnode_tnodestatus_list_t;
#endif


class SSNode: public MDDNode {
public:
	SSNode(short _var, short _size) {
		Construct(_var, _size);
	}
	SSNode(short _var, short _size, MDDNode* _initial_value) {
		Construct(_var, _size, _initial_value);
	}
	SSNode(short _val) {
		Construct(_val);
	}
	SSNode(SSNode* v);
	SSNode() {
	}
	;
	virtual ~SSNode() {
		ResetStateOffsets();
	}

	void Construct(short _var, short _size);
	void Construct(short _var, short _size, MDDNode* _initial_value);
	void Construct(short _val);
	void initializeNode();

	void setPointer(unsigned short i, MDDNode* node) {
		check_limits(i);
		MDDNode::setPointer(i, node);
	}
	void setPointerN(unsigned short i, MDDNode* node) {
		check_limitsN(i);
		MDDNode::setPointerN(i, node);
	}
	void setPointerToInitialValue(unsigned short i) {
		MDDNode::setPointer(i, initial_value);
	}
	void assign(unsigned short i, MDDNode* node) {
		check_limitsN(i);
		d[i] = node;
	}

	MDDNode* getPointer(unsigned short i) const {
		if (i >= size)
			return initial_value;
		else
			return d[i];
	}

	virtual void print(bool detailed = true);

	bool pointsToNull(unsigned short i) const {
		return d[i] == initial_value;
	}

	virtual int operator==(const MDDNode& a);

	void ComputeStateOffsets();
	void ComputeStateOffsets(ssnode_list_t& list);
	INDEX_TYPE ComputeStateOffsetsAux(ssnode_list_t& list);
	void ResetStateOffsets();

	INDEX_TYPE getStateOffset(short i) {
		assert(i <= sparse_size);
		return state_offset[i];
	}

	MDDNode* getdAct(short act_substate) {
		return d[sparse2fullindex[act_substate]];
	}

	short ActSubstate2PotSubstate(short actual_substate) {
		assert(actual_substate <= sparse_size);
		return sparse2fullindex[actual_substate];
	}

	short PotSubstate2ActSubstate(short pot_substate) {
		assert(pot_substate < size);
		assert(full2sparseindex[pot_substate] != -1);
		return full2sparseindex[pot_substate];
	}

	INDEX_TYPE PotSubstate2StateOffset(short pot_substate) {
		return getStateOffset(PotSubstate2ActSubstate(pot_substate));
		//state_offset[full2sparseindex[pot_substate]];
	}

	short StateOffset2Substate(INDEX_TYPE state_offset);

	INDEX_TYPE getNStates() {
		return state_offset[sparse_size];
	}

	INDEX_TYPE Path2StateOffset(PATH_TYPE* path);

	short getSparseSize() {
		return sparse_size;
	}

	int SizeInMemory() {
		int result = 12 * 4;
		if (var >= 0) {
			result += size * 4 * 3;
		}
		return result;
	}

	void TreeInfo(pair<int, int>& p, bool show);
	void PrintTree();
	void print_dag();
	void print_dag_aux(ssnode_tnodestatus_list_t& list);

	TNodeStatus status;
	//A pointer to the node which is a "useful" copy of the current node. This
	//value is used as a cache in method CopyUsefulNodes.
	//This value is cleared after CopyUsefulNodes is finished. This technique of
	//caching saves a huge amount of time.
	SSNode* UsefulCopy;
	MDDNode *initial_value;

protected:

	void extend_size(unsigned short new_index);
	void check_limits(unsigned short i);
	void check_limitsN(unsigned short i);
	INDEX_TYPE* state_offset;
	short* sparse2fullindex; //converts sparse_position to full_position
	short* full2sparseindex; //converts full_position to sparse_position
	//meaningless when applied to a position which does not
	//have a corresponding sparse_position
	short sparse_size;

	short capacity;
};

void ResetStateOffsets(ssnode_list_t& l);

#endif
