/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/** \class Permutation
 *
 * Represents a single permutation.
 *
 * In this case, it is an automorphism. The permutation is
 * stored as an array of integers that map from the original
 * coordinate to the new coordinate.
 *
 * \author Michael McQuinn \date March, 2006
 */

#include <iostream>
#include <vector>
#include <cstdlib>
using namespace std;

#ifndef PERMUTATION
#define PERMUTATION

class Permutation {

public:
	Permutation();
	Permutation(int theN);
	Permutation(int theN, int* theP);
	Permutation(const Permutation &perm);
	Permutation(vector<int> tempPerm);
	~Permutation();

	/* Accessors */
	int GetN();
	void SetN(int new_n) {
		n = new_n;
	}
	;
	int* GetP();
	int GetIth(int i);
	bool IsIdentity();

	/* Overloaded operators */
	bool operator ==(Permutation param);
	bool operator =(Permutation param);
	int operator [](int col);
	friend ostream & operator <<(ostream &os, const Permutation &param);

private:

	/** Number of elements in the Permutation. */
	int n;

	/** An array of permutation elements */
	int* p;
};

#endif
