/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#include <iostream>

#ifndef SCHREIER_SIMS_P_H
#define SCHREIER_SIMS_P_H

using namespace std;
const int maxn = 200; // Maximum size of omega = {1, ,n}
const int maxr = 500; // Maximum number of generators for the group


class SchreierSimsPermutation { // interface for permutations
public:
	int p[maxn]; // the images of the points 0..maxn-1
	SchreierSimsPermutation() {
		n = maxn;
	}
	;
	SchreierSimsPermutation(int m) {
		n = m;
	}
	;
	SchreierSimsPermutation(int m, char c) {
		SetDefault(m, c);
	}
	;
	void SetDefault(int m, char c) {
		n = m;
		switch (c) {
		case 'i':
			for (int i = 0; i < n; i++)
				p[i] = i;
			break;
		default:
			for (int i = 0; i < n; i++)
				p[i] = -1;
			break;
		}
	}
	SchreierSimsPermutation operator *(SchreierSimsPermutation param) const {
		SchreierSimsPermutation result(n);
		for (int i = 0; i < n; i++)
			result.p[i] = param.p[p[i]];
		return (result);
	}
	;
	void operator *=(SchreierSimsPermutation param) { // direct multiplication
		for (int i = 0; i < n; i++)
			p[i] = param.p[p[i]];
	}
	;
	SchreierSimsPermutation inverse() const { // inverse
		SchreierSimsPermutation result(n);
		for (int i = 0; i < n; i++)
			result.p[p[i]] = i;
		return (result);
	}
	;
	bool isdefined() const {
		return (p[0] > -1);
	}
	; // if it is defined
	bool isidentity() const { // if it is the identity
		for (int i = 0; i < n; i++)
			if (p[i] != i)
				return false;
		return true;
	}
	;
	bool operator ==(SchreierSimsPermutation param) const {
		for (int i = 0; i < n; i++)
			if (param.p[i] != p[i])
				return false;
		return true;
	}
	;
	void input() {
		for (int i = 0; i < n; i++) {
			cin >> p[i];
			p[i]--;
		}
	}
	;
	void output() const {
		for (int i = 0; i < n; i++)
			cout << p[i] + 1 << " ";
		cout << endl;
	}
	;
	void setn(int m) {
		n = m;
	}
	;
	int fbp; // to store the first basepoint moved by this permutation
private:
	int n;
};

#endif
