/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/** \class SchreierTree
 *
 * Schreier Tree implementation.
 *
 * A tree of SchreierTreeNodes.  Supports adding children,
 * and searching for nodes in a couple different ways.
 *
 * \author Michael McQuinn \date March, 2006
 */
#include "SchreierTreeNode.h"
#include "ValPerms.h"
#include <string>

class SchreierTree {

public:
	SchreierTree();
	SchreierTree(int theN, int theAlpha);
	~SchreierTree();

	SchreierTreeNode* Find(Permutation theP);
	SchreierTreeNode* Find(int theValue);
	void AddChild(Permutation findP, int theValue, Permutation theP);
	vector<Permutation> GetPermutationsToRoot(int theValue);
	void SetRootValue(int theValue);
	vector<ValPerms> GetAllPermutationsToRoot();

	/* Tree Operations */
	void ReadTree(string FileName);
	void ClearTree();
	void CreateTree(int theValue);

	/* Debugging */
	void PrintTree();
	void PrintPerms();

	/* Accessors */
	int GetN();

	void SetN(int _n) {
		n = _n;
	}
	void SetPerms(vector<Permutation> permVec);

private:

	/** The root of the SchreierTree */
	SchreierTreeNode* root;

	/** The number of elements in a permutation */
	int n;

	/** A vector holding all of the Permutations */
	vector<Permutation*> Perms;

	/** The orbit of the Schreier Tree
	 * \todo Can I get rid of this? */
	int alpha;

};
