/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/** A vertex of the Schreier Tree.
 *
 *  Stores a vector of children, a parent pointers, a value
 *  (representing the basis element at this vertex), and the
 *  Permutation on the edge from this vertex to its parent.
 *
 * \author Michael McQuinn \date March, 2006
 */
#include <vector>
#include "Permutation.h"
#include "ValPerms.h"

class SchreierTreeNode {

public:
	SchreierTreeNode();
	SchreierTreeNode(int theN, int theValue);
	SchreierTreeNode(int theValue, Permutation theP);
	~SchreierTreeNode();

	void AddChild(int theValue, Permutation theP);
	void PrintNode(int tabLevel);
	SchreierTreeNode* Find(Permutation P);
	SchreierTreeNode* Find(int theValue);
	void GetPermutationsToRoot(vector<Permutation> & vec);
	void GetAllPermutationsToRoot(vector<ValPerms> & vec);

	/* Node Operations */
	void ClearNode();

	int GetValue();
	Permutation GetPermutation();
	int GetNumChildren();

private:

	/** Pointer to the node's parent */
	SchreierTreeNode* parent;

	/** Children pointers */
	std::vector<SchreierTreeNode*> children;

	/** The permutation from this node to its parent */
	Permutation *p;

	/** The value of the node, ie the basis element represented at this node
	 *
	 */
	int value;

	/** The number of elements in a permutation */
	int n;
};
