/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/
#ifndef SYMMETRY_DETECTION_H
#define SYMMETRY_DETECTION_H

#include "GraphComposer/Wrapper.h"
#include "ModelCompGraphNode.h"
#include "ModelCompGraphEdge.h"
#include "saucy.h"
#include "ValPerms.h"
#include <string>
#include <vector>
#include <map>


class SymmetryDetection {
public:
	SymmetryDetection();
	SymmetryDetection(Wrapper* _wrapper);
	void FindMCG();
	void PrintGraphNodes();
	void PrintGraphEdges();
	void CreatePartitionInfo();
	void PrintPartitionInfo();
	void AdjustPermutations();
	void ShrinkPermutations();
	void AdjustPartitionInfo();

	/** Removes identity Permutations and 'lumps' permutations */
	void TrimPermutations();

	int GetTotalStateSize();
	int DetermineTotalStateSize();

	/** Lots of initializes */
	void Initialize();

	/** Lots of cleanup */
	void Cleanup();

	void FindCanonicalLabel(void * state, void* state_prime, short* best_perm = NULL);
	void WritePermFile(string & file_name);
	void ReadPermFile(string & file_name);
	std::vector<int>* af_orbit;
	std::vector<int>* af_k;
	short* af_ident_perm;
	short* af_perm;
	short* af_best_perm;
	short* af_temp_best_perm;
	short* af_iter_best_perm;

	std::vector<ModelCompGraphNode> GraphNodes;

	bool IsPartiallyCanonical(int level, void* state);
	std::vector<State*>* StateSpace;

	/** Different ordering of SVs.
	 *
	 * Used to adjust permutations to state space order */
	std::vector<string> GraphNodeOrder;
	std::vector<string> SSOrder;

	/** Used a lot, so I made them public */
	std::vector<int> bases;
	std::vector<int> PartitionInfo; // State Size in bytes
	std::vector<int> CumPartitionInfo; // Cum offset in bytes
	std::vector<int> StaticCumPartitionInfo; // static version

	// hack-ish
	byte* is_bigger_state;
	byte* biggest_state;
	byte* iter_state;
	byte* temp_state; // FindNewState
	byte* temp_state2;// FindNewState

	void PrintState(void* State, int NumBytes);
	void WriteState(void* State, int NumBytes, FILE* f);
	void PrintPermutations();

	/** Final symmetry data
	 *
	 * \todo Move to private
	 */
	std::vector<int> F_Bases;
	std::map<int, std::vector<ValPerms> > RootPermutations;

	/** Static members */
	int st_num_bases;
	int * st_bases;
	int * st_cum_part_info;
	int * st_part_info;
	ValPerms** st_root_permutations;
	int * st_num_valperms;
	int * st_orbit;
	int * st_k;

	/** Num Levels, might need moving */
	int NumLevels;

private:
	Wrapper* wrap;
	BaseGraphComposerClass** JoinArray;
	int NumJoins;
	std::vector<ModelCompGraphEdge> GraphEdges;
	std::string FindType(std::string name);
	int FindIndex(std::string name);

	/** might be redundant from GraphSymSSG */
	int TotalStateSize;

	/** Used to initialize some MCG things */
	saucy_obj * my_saucy;
	void InitFindMCG();

	void FindMCG_JoinNodes();
	void FindMCG_PrivateNodes();
	void FindMCG_Sort();
	void FindMCG_FindAut();
	void FindMCG_FindSS();
	void FindMCG_FindST();
	void FindGraphNodeOrder();
	void FindSSOrder();
	void SwapNodes(int i, int j);

	/** Helper method to strip string before "->" in join name */
	std::string GetJoinName(std::string join_name);

	/** Prints the MCG to file */
	void PrintMCG();
	void WriteMCGViz();
	std::vector<std::string> types;
	std::vector<Permutation> SGS;
	std::vector<Permutation> OrigPermutations;

	/** Helper Methods to adjust permutations */
	int FindIndexSSOrder(std::string name);
	int FindIndexGNOrder(std::string name);

	/** FindCanonicalLabel Helpers */
	void FindOrbit(int i, std::vector<int> *orbit);
	int FindOrbit_static(int i, int* orbit);
	void FindK(void *State, std::vector<int>& orbit, std::vector<int>& K, int size);
	void* FindNewState(void* State, int base, int orbit, void* StateCopy);
	void* FindNewState_static(void* State, int base, int orbit, void* StateCopy);

public:
	void ConvertSTLToStatic();
	void CleanStaticMembers();
	void PrintStaticMembers();
};

#endif
