/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: Sync.h
 *
 *      Authors: Aaron Stillman, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef SYNC_H_
#define SYNC_H_

#include "BaseComposerClass.h"
#include "../BaseClasses/List.h"
#include "../BaseClasses/MobiusEnums.h"
#include "../BaseClasses/BaseSyncActionClass.h"
//#include "../BaseClasses/BaseActionClass.h"

#include <vector>


class Sync: public BaseComposerClass {
public:
	/** The sync constructor
	 *
	 * @param theName The name given to the sync node
	 * @param numModels the number of submodels
	 * @param numActions the number of synchronizing actions
	 * @param number of SVs to be exported
	 * @param names of SVs to be exported
	 *
	 */
	Sync(char*, int, int, int, char**);

	~Sync();
	BaseSyncActionClass** SyncActionList;

	// Variables of each Instance
	int NumofSuperposingActions;

	// This methods removes superposed Actions from ActionList
	// of the node:
	// @param: the superposing Action to be inserted instead
	void CleanGroupListFromSuperposed(BaseSyncActionClass*);
	void CleanActionListFromSuperposed(BaseSyncActionClass*);

	// This methods removes superposed Actions from GroupList
	// of the node:
	// @param: the superposing Action to be inserted instead
	void ReSetGroups(BaseSyncActionClass*);

	//bool CompareState(void* , void* );

	virtual int Rank(BaseSyncActionClass *) {
		return 0;
	}
	;
	virtual double Weight(BaseSyncActionClass *) {
		return 0.0;
	}
	;
	virtual double Rate(BaseSyncActionClass *) {
		return 0.0;
	}
	;

	// Sets the number of Shared SV, required for
	// exporting SV from Sync nodes
	//////////////////////////////////////////////
	inline void setNumSharedStateVariables(int i) {
		NumSharedStateVariables = i;
	}
	;
	inline int getNumofSuperposingAction() {
		return (NumofSuperposingActions);
	}
	;
	inline int getNumSuperposedActions(int i) {
		return (SyncActionList[i]->getNumSuperposedActions());
	}
	;
	inline BaseActionClass** getSuperposedActionList(int i) {
		return (SyncActionList[i]->getSuperposedActionList());
	}
	;
	inline BaseActionClass* getSuperposingAction(int i) {
		return (SyncActionList[i]);
	}
	;

	///////////////////////////////
	// Unchanged taken from Join://
	///////////////////////////////
	int * ActionNameCardinalities;
	int * SubmodelStateSize;
	bool LookForward;
	unsigned int currentStateIndex, currentStateCounter;
	unsigned int stateSizeCounter, stateSizeTotal;
	unsigned int hashActionsCounter, hashActionsIndex;
	unsigned int numActionsCounter, numActionsIndex;
	unsigned int numModelsCounter, numModelsIndex;
	unsigned int setStateIndex, setStateCounter;
	virtual int StateSize();
	virtual void SetState(void * TheAddress);
	void Setup();
	void HashActions();
	virtual void CurrentState(void * TheLocation);

protected:
	int ActionNameTableSize;
//	char ** ActionNameHashTable;
	std::vector<const char*> ActionNameHashTable;
};
#endif
