/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: ImpulseNodeClass.hpp
 *
 *      Authors: Gerard P. Kavanaugh, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef PV_IMPULSE_HPP_
#define PV_IMPULSE_HPP_

#include "../BaseClasses/BaseModelClass.h"
#include "../BaseClasses/BaseActionClass.h"
#include <string.h>

// I know putting the following two functions in here is not the good way
// of doing it. I just temporarily put them in here until we decide about
// a file that we put these kinds of functions in.

#ifndef COMMON
#define COMMON
template<class T>
inline T Max(T a, T b) {
	if (a > b)
		return a;
	else
		return b;
}

template<class T>
inline T Min(T a, T b) {
	if (a < b)
		return a;
	else
		return b;
}

template<class T>
inline T CommonInterval(T x1, T x2, T y1, T y2) {
	return Max(0.0, Min(x2, y2) - Max(x1, y1));
}

#endif

/**
 *  This is the base class for all
 *  performance variable impulse
 *  rewards.
 *
 *  @author Gerard P. Kavanaugh
 *  @author Jay M. Doyle
 *  @version 1.0
 */
class ImpulseNodeClass {

public:

	/**
	 *
	 */
	BaseModelClass ***TheModelPtr;
	/**
	 *
	 */
	BaseModelClass *ContainingModel;
	/**
	 *
	 */
	char *Name;

	/**
	 *
	 */
	int Trace;

	/**
	 *
	 */
	int WorkerNumber;

	/**
	 *
	 */
	int ImpulseNumber;

	/**
	 *
	 */
	int NumberOfModelDependencies;

	/**
	 *
	 */
	int id;

	/**
	 *
	 */
	double Impulse;

	/**
	 *Squared Impulse Reward
	 */
	double ImpulseSqr;
	double ImpulseFraction;

	/**
	 *
	 */
	double Start;

	/**
	 *
	 */
	double Stop;

	ImpulseNodeClass(void);
	virtual ~ImpulseNodeClass(void);
	virtual void completeAccumulation(double) = 0;
	virtual double Impulse_Function(double) = 0;
	void FixReferences(BaseModelClass*);
	virtual void Accumulate(double, double) = 0;

	//    virtual void EndAccumulate(double,double) = 0;

	/**
	 *
	 */
	double min(double, double);

	/**
	 *
	 */
	double max(double, double);

	/**
	 *
	 */
	virtual void CurrentImpulse(void) = 0;

	/**
	 *
	 */
	virtual void Reset(void) = 0;

	/**
	 *
	 */
	void DefineName(char*, int, int);

	/**
	 *
	 */
	virtual ImpulseNodeClass** CreateImpulseWorkerList(int) = 0;

}; // end ImpulseNodeClass class


#endif //PV_IMPULSE_HPP_
