/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: InstantOfTime.hpp
 *
 *      Authors: Gerard P. Kavanaugh, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef INSTANT_OF_TIME_HPP_
#define INSTANT_OF_TIME_HPP_

#include "PerformanceVariableWorker.hpp"
#include "../BaseClasses/BaseActionClass.h"

/**
 *  This class defines the PerformanceVariableWorker
 *  implementation for instant of time performance
 *  variable.  An instant of time performance variable
 *  peformance variable worker is responsible for calculating
 *  the appropriate reward value at a specific instant in
 *  time.
 *
 *  @author Gerard P. Kavanaugh
 *  @author Jay M. Doyle
 *  @version 1.0
 */
class InstantOfTime: public PerformanceVariableWorker {

public:

	/**
	 *  Default constructor
	 */
	InstantOfTime();

	/**
	 *  Destructor
	 */
	~InstantOfTime();

	/**
	 *  This is the normal accumulate method that accumulate
	 *  reward based upon the current simulation time and
	 *  and the last event time.
	 *
	 *  @param new_time The current time of the simulation
	 *  @param last_event_time The time of the previous
	 *          simulation event
	 */
	void Accumulate(double new_time, double last_event_time);

	/**
	 *  This calculates the current reward
	 *  of the performance variable
	 */
	void CurrentReward(void);

	//virtual double Reward_Function(void){cerr<<"InstantOfTime::Reward_Function"<<endl;}

	//void testMethod(){cout <<"** InstantOfTime::testMethod()"<<endl;}

}; // end InstantOfTime class

#endif //INSTANT_OF_TIME_HPP_
