/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: DataPacket.h
 *
 *      Authors: Tod Courtney, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 9-13-2000
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef _DATAPACKET_H_
#define _DATAPACKET_H_

// create the report data packet
class DataPacket {
private:
	int _clientId;
	int _experimentNumber;
	int _numPV;
	int _maxNumReports;
	int *_numObs;
	double *_observations;

public:
	DataPacket(int ClientID, int ExperimentNumber, int NumPV, int NumReports);
	~DataPacket();
	inline int GetClientId() const;
	inline int GetExperimentNum() const;
	inline int GetNumObs(int pv) const;
	inline double GetObs(int pv, int reportNum) const;
	inline double *GetObsArray(int pv) const;
	inline void SetClientId(int id);
	inline void SetExperimentNum(int expNum);
	inline void SetObs(int pv, int reportNum, double value);
	inline void SetNumObs(int pv, int num);
};

inline int DataPacket::GetClientId() const {
	return _clientId;
}
inline int DataPacket::GetExperimentNum() const {
	return _experimentNumber;
}

inline int DataPacket::GetNumObs(int pv) const {
	return _numObs[pv];
}

inline double DataPacket::GetObs(int pv, int reportNum) const {
	return _observations[pv * _maxNumReports + reportNum];
}
inline double *DataPacket::GetObsArray(int pv) const {
	return &(_observations[pv * _maxNumReports]);
}

inline void DataPacket::SetClientId(int clientId) {
	_clientId = clientId;
}
inline void DataPacket::SetExperimentNum(int expNum) {
	_experimentNumber = expNum;
}
inline void DataPacket::SetNumObs(int pv, int num) {
	_numObs[pv] = num;
}
inline void DataPacket::SetObs(int pv, int reportNum, double value) {
	_observations[pv * _maxNumReports + reportNum] = value;
}

#endif
