/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: Event.h
 *
 *      Authors: Alex Williamson, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef EVENT_CLASS
#define EVENT_CLASS

#include "../BaseClasses/BaseActionClass.h"

/**
 * This class is the base class for events in the Mobius simulator.
 * Specific event classes for the different event list types should
 * be derived from this class
 *
 * @author Alex Williamson
 * @version Mobius v 1.0
 */

class Event {
public:
	~Event();
	Event();

	/**
	 * Overloaded constructor
	 * @param double the event execution time
	 * @param double the event start time
	 * @param BaseActionClass* Pointer to action associated with event
	 */
	Event(double execTime, double startTime, BaseActionClass*);

	/**
	 * Overload the copy constructor and assignment "=" operator
	 */
	Event(Event const & other);
	Event const &operator=(Event const &other);

	/**
	 * Execution time of event
	 */
	double exec_time;
	/**
	 * Start time of event
	 */
	double start_time;
	/**
	 * Pointer to action
	 */
	BaseActionClass* action_ptr;
};

/* These two function objects (functors) are used to compare
 two events in the FEL(future event list) which is implemented
 as a heap data structure
 */

// An event is supposed to be taken off the heap earlier than others
// if its execution time is the earliest or
// (in the case of equal execution time) if it has the highest rank
struct EventEarlier {
	bool operator()(Event* e1, Event* e2) {
		return (e1->exec_time < e2->exec_time) || ((e1->exec_time == e2->exec_time) && (e1->action_ptr->Rank() > e2->action_ptr->Rank()));
	}
};

struct EventEqual {
	bool operator()(Event* e1, Event* e2) {
		return (e1->exec_time == e2->exec_time) && (e1->action_ptr->Rank() == e2->action_ptr->Rank());
	}
};

#endif
