/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: LLEventList.h
 *
 *      Authors: Alex Williamson, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef LLEVENTLIST_CLASS
#define LLEVENTLIST_CLASS

#include "../BaseClasses/BaseActionClass.h"
#include "EventList.h"
#include "Event.h"
#include <stddef.h>

/**
 * A derived event class to implement a linked list events list.
 *
 * @author Alex Williamson
 * @version Mobius v 1.0
 */
class LLEvent: public Event {
public:
	~LLEvent();
	LLEvent();

	/**
	 * Specifies if the event has event connected on the rank/weight links.
	 * Essential acts as a boolean.
	 */
	short multiple;
	/**
	 * Next event on the events list.
	 */
	LLEvent* next;
	/**
	 * Previous event on the events list.
	 */
	LLEvent* prev;
	/**
	 * Next event with the same execution time.
	 */
	LLEvent* nextweight;
	/**
	 * Previous event with the same execution time.
	 */
	LLEvent* prevweight;
};

/**
 * This is a derived event list class used to implement a linked
 * list events list. See EventList.h for method descriptions.
 * @see EventList
 *
 * @author Alex Williamson
 * @version Mobius v 1.0
 */
class LLEventList: public EventList {
public:
	~LLEventList();
	LLEventList();
	LLEventList(int);

	/**
	 * Head of the event list.
	 */
	LLEvent* head;
	/**
	 * Tail of the event list (unused).
	 */
	LLEvent* tail;
	/**
	 * Head of the free list.
	 */
	LLEvent* freehead;
	inline void ClearEventList();
	inline void Insert(Event*); // return 1 on error, 0 otherwise
	inline void Remove(Event*);
	inline Event* Earliest();
	void PrintList();
	int EventListLength();
	inline void FreePush(Event*);
	inline Event* FreePop();

	// Methods to save event list and reset
	void* SaveEventList();
	short SetEventList(void*, void*);

	LLEvent* NewEvent;
	LLEvent* CurrEvent;
	LLEvent* FreeTemp;
	double Weight1, Weight2;
	int Rank1, Rank2;
	double weightsum, temprnd;

};

#endif
