/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: LaggedFibonacci.h
 *
 *      Authors: Alex Williamson, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef LAGGEDFIBONACCI
#define LAGGEDFIBONACCI

#include "RandomNumberGenerator.h"
#define numToGen 100
/**
 * Class derived from RandomNumberGenerator to implement the Knuth
 * Lagged Fibonacci generator.  Calls external function in
 * Rnd-KnuthLaggedFibonacci.c. See RandomNumberGenerator for
 * method descriptions.
 * @see RandomNumberGenerator
 *
 * @author Alex Williamson
 * @version Mobius v 1.0
 */
class LaggedFibonacci: public RandomNumberGenerator {
public:
	LaggedFibonacci();
	~LaggedFibonacci()=default;

	double rnd() override;
	void srnd(unsigned long) override;

	/**
	 * Double array where random number are stored.
	 */
	double rndArr[numToGen];
	private:
		int k;
};

#endif
