/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: ReactivationList.h
 *
 *      Authors: Alex Williamson, David Daly, Daniel D. Deavours, ,
 *                William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef REACTIVATIONLIST
#define REACTIVATIONLIST

// The nubmer of ReactivationListElement
// object created if the free list becomes empty
#define REACTIVATION_BLOCK_SIZE 10

#include "../BaseClasses/BaseActionClass.h"

/**
 * Element of ReactivationList.  Implements simple singly linked
 * list structure.
 * @see ReactivationList
 *
 * @author Alex Williamson
 * @author David Daly
 * @verion Mobius v 1.0
 */
class ReactivationListElement {
public:
	ReactivationListElement();
	~ReactivationListElement();

	/**
	 * Next element in list.
	 */
	ReactivationListElement* next;
	/**
	 * Pointer to representative action.
	 */
	BaseActionClass* action;
};

/**
 * A simple singly linked list structure to act as a stack for reactivatable
 * action meeting the criteria specified in "Discrete Event Simulation in the
 * Mobius Modeling Framework".
 *
 * @author Alex Williamson
 * @version Mobius v 1.0
 */
class ReactivationList {
public:
	ReactivationList();
	/**
	 * Constructor to specify the max number of reactivatable actions.
	 * @param Maximum number of reactivatable actions.
	 */
	ReactivationList(long);
	~ReactivationList();

	/**
	 * Pointer to head of stack.
	 */
	ReactivationListElement* head;

	/**
	 * Pointer to head of free stack.
	 */
	ReactivationListElement* freehead;

	/**
	 * Temporary pointer.
	 */
	ReactivationListElement* Temp;

	/**
	 * Insert an action into the stack.
	 * @param BaseActionClas* The action to be inserted.
	 */
	void Insert(BaseActionClass*);
	/**
	 * Pop top element off stack and place of free list.
	 */
	void Remove();

	/**
	 * Resets the iterator to the head of the list. The next call to
	 * next() *after a call to resetIterator will return the head
	 * element.
	 */
	void resetIterator();

	/**
	 * Returned the next element in the list, or null if at the end.
	 */
	ReactivationListElement* next();

	/**
	 * Removed the current element from the linked list.
	 */
	void removeCurrElement();

protected:

	/**
	 * Points to the current element in the iteration
	 */
	ReactivationListElement* Curr;

	/**
	 * Points to the previous element in the iteration.
	 */
	ReactivationListElement* Prev;

};

#endif
