/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#include <iostream>
#include <math.h>
#include <vector>
#include <stdio.h>
#include "StateLevelObjects/BaseStateLevel.h"
#include "StateLevelSolvers/common/BaseStateLevelSolver.h"
#include "StateLevelSolvers/common/solvers.h"
#include "StateLevelSolvers/common/common_db.h"

class ARSSolver: public BaseStateLevelSolver {
public:
	typedef vector<RateType> VectorType;

	ARSSolver(BaseStateLevel *_model, int argc, char** argv);
	~ARSSolver();
	void initialize();
	void solve();

protected:
	virtual void ShowCmdLineOptions();

private:
	void add_time_interval(char* paramstr);
	void DisplayBanners();
	void terminate();
	void output(double lambda, int nonzero_rewards, unsigned int* l_trunc_points, unsigned int* r_trunc_points, double* trunc_error);
	void updatedb();
	void computePVs();
	void getUnionTimes(BaseStateLevel *model, vector<double> &start, vector<double> &end);

	RateType epsilon;

	typedef struct {
		RateType left;
		RateType right;
	} Interval;

	int max_intervals;
	int no_of_intervals;
	Interval* intervals; //The time intervals in which to compute the prob. dist.
	int no_of_unique_tpts;
	RateType* unique_tpts;
	int* timeptsidx_to_uniqueidx;

	RateType* trunc_error;
	RateType** solutions; /* array of solution vectors; a vector for each time point*/
	RateType** solutions_impulse;

	bool has_impulse_reward;
	RateType lambda, inv_lambda;
	bool UsePVType;
};
