/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#include <iostream>
#include <math.h>
#include <vector>
#include <stdio.h>
#include "StateLevelObjects/BaseStateLevel.h"
#include "StateLevelSolvers/common/BaseStateLevelSolver.h"
#include "StateLevelSolvers/common/solvers.h"
#include "StateLevelSolvers/common/common_db.h"

class ATSSolver: public BaseStateLevelSolver {
public:
	typedef vector<RateType> VectorType;

	ATSSolver(BaseStateLevel *_model, int argc, char** argv);
	~ATSSolver();
	void initialize();
	void solve();

	RateType getSolutionVectorElement(int time_point, SizeType s);
	void getSolutionVectorAsArray(int time_point, RateType*& vec);
protected:
	virtual void ShowCmdLineOptions();
	void print_states();
	void getUnionTimes(BaseStateLevel *model, vector<double> &times);

private:
	void DisplayBanners();
	unsigned long find_bp();
	RateType find_autime(RateType sw_prob, unsigned long bplength);
	void bp_gen(unsigned long bplength, RateType& act_acc_SOT, RateType* pvec, RateType time);
	void terminate();
	void output(double lambda, double swtch, unsigned long conv_epoch, double act_acc_LU, double *act_acc_tot, unsigned int* r_trunc_points);
	void mat_mult(double *x, double *y, double *z, unsigned long bplength);
	void updatedb();

	RateType overall_epsilon;
	RateType SU_epsilon;
	RateType LU_epsilon;
	RateType SOTprob;
	RateType fracAU;
	RateType step_yoon;
	RateType lambda;
	bool dofrac;
	int numrates;
	bool comp_flag;
	bool UsePVTypes;

	int no_of_time_points;
	RateType* time_points; //The time points in which to compute the prob. dist.
	RateType* diag;

	RateType* prob_vec; /* current DTMC probability vector */
	RateType* prev_prob_vec; /* previous DTMC probablity vector */
	RateType** solutions; /* array of solution vectors; a vector for each time point*/

	RateType *lambda_t; /* multiplication of lambda and each time point */
	RateType **poisson_coef;
	unsigned int *l_trunc_points, *r_trunc_points;

	RateType* bprates;

	BaseStateLevel::EdgeBaseIterator* it;
	BaseStateLevel::EdgeBaseContainer* All;
};
