/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef _BASESTATELEVELSOLVER_H_
#define _BASESTATELEVELSOLVER_H_

class BaseStateLevelSolver;

#include "common_db.h"
#include "solvers.h"
#include <string>

using namespace std;

inline void setMacServerInfo(char *) {
}

class BaseStateLevelSolver {
public:
	static BaseStateLevelSolver*
	createStateLevelSolver(BaseStateLevel *lts, int argc, char**argv);

	BaseStateLevelSolver(BaseStateLevel *_model, const char* _solver_name, int argc, char** argv);

	virtual ~BaseStateLevelSolver();
	virtual void solve() = 0;
	virtual void initialize();

	const char* getSolverName() {
		return solver_name;
	}
	const char* getSolverFullPath() {
		return solver_full_path;
	}
	const char* getSSGFullPath() {
		return ssg_full_path;
	}
	const char* getProjectName() {
		return project_name;
	}
	const char* getStudyName() {
		return study_name;
	}
	const char* getExperimentName() {
		return experiment_name;
	}

protected:

	char* solver_name;
	BaseStateLevel* model;
	char* ssg_full_path;
	char* solver_full_path;
	char* project_name;
	char* study_name;
	char* experiment_name;
	char* debug_filename;
	char* output_filename;
	char* csv_filename;
	string comments_filename;
	bool debug_flag;
	bool output_flag;
	bool states_flag;
	char* states_filename;
	int verbosity;

	FILE* fout;
	FILE* fcsv;
	FILE* fdebug;
	FILE* fstates;
	double cputime, systime; //Time taken to solve the modle
	SolverDBInfo *dbDataPtr;
	static RateType machine_epsilon;

	void initializeDatabaseData();
	void ShowCommonCmdLineOptions(const char* exec_name);
	virtual void ShowCmdLineOptions() = 0;
};
#endif
