/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef COMMON_DB__H
#define COMMON_DB__H

#include <stdio.h>
#include "StateLevelObjects/BaseStateLevel.h"

#define COLLECT_PDF 1  /* collect pdf for each pv, can be very slow for PV
                          with a lot of states */

// TBD: remove these after port to new database is done
#define DB_ENABLED 1
#define DataSet char
#define DBbool char
#define MAC_FALSE 0
#define MAC_TRUE 1

/* name of the database entity to write results to */
extern char db_entity[81];
/* host name/port of the Mac db server */
extern char db_host[81];
/* index of experiment entry in the database record */
extern int db_exp_index;

void initDBGlobals();
void parseDBSwitches(char *str);
char *packageArrayElement(DataSet *ds, char *array_name, char *array_var, int element, char *type);

/* this structure represents the results generated
 * for a single performance variable
 */
typedef struct {
	char entity[64];
	double mean;
	double variance;
	double *pdf_x;
	double *pdf_y;
	int pdf_size;
} SolverPV;

/* this structure represents the results generated
 * for a single ARS performance variable
 */
typedef struct {
	char entity[64];
	double mean;
	double time_ave_mean;
} ARSSolverPV;

void initPV(char *db_entity, int db_exp_index, BaseStateLevel* model, const char *exp_name, int time_mode, int num_times);
void writeSolverPV(SolverPV *data, int exper_index, int time_mode, int time_index, double time_pt, int pv_index);

void writeARSSolverPV(ARSSolverPV *data, int exper_index, int time_index, double time_start, double time_end, int pv_index);

/* for now, just hard limit these.  There is only 1 of these per
 * solver, so the size isn't a big deal.
 */
#define DBMAXARRAYSIZE 8
#define DBMAXNAMESIZE 25
typedef struct {
	int NumInt8;
	char NameInt8[DBMAXARRAYSIZE][DBMAXNAMESIZE];
	long ValueInt8[DBMAXARRAYSIZE];

	int NumInt4;
	char NameInt4[DBMAXARRAYSIZE][DBMAXNAMESIZE];
	long ValueInt4[DBMAXARRAYSIZE];

	int NumFloat8;
	char NameFloat8[DBMAXARRAYSIZE][DBMAXNAMESIZE];
	double ValueFloat8[DBMAXARRAYSIZE];

	int NumBoolean;
	char NameBoolean[DBMAXARRAYSIZE][DBMAXNAMESIZE];
	DBbool ValueBoolean[DBMAXARRAYSIZE];
} SolverDBInfo;

void SolverDBInfo_Init(SolverDBInfo *data);
int SolverDBInfo_GetTotalNum(SolverDBInfo *data);
void SolverDBInfo_AddInt8(SolverDBInfo *data, char *name, long value);
void SolverDBInfo_AddInt4(SolverDBInfo *data, char *name, long value);
void SolverDBInfo_AddFloat8(SolverDBInfo *data, char *name, double value);
void SolverDBInfo_AddBoolean(SolverDBInfo *data, char *name, DBbool value);

void initSolverDB(char *db_entity, int db_exp_index, long s, long nonzero_elements);
void SolverDBInfo_Write(SolverDBInfo *data, char *db_entity, int exper_index);

#endif

/* this are replaced by SolverDBInfo objects and methods, but saved
 * for future reference */
#if 0
typedef struct {
	long iter;
	long total_time;
	long usr_sec;
	long sys_sec;
	DBbool did_converge;
	double max_diff;
}ISSSolverExperInfo;

typedef struct {
	long total_time;
	long usr_sec;
	long sys_sec;
	DBbool did_converge;
	double max_diff;
	double drop_tol;
	long non_zeros_factor;
	int correct_digits;
	long zeros_factor;
	long elem_dropped;
	long new_pivots;
}DSSSolverExperInfo;

extern void writeISSSolverExperInfo( ISSSolverExperInfo *data,
		char *db_entity, int exper_index);
extern void writeDSSSolverExperInfo( DSSSolverExperInfo *data,
		char *db_entity, int exper_index);
#endif
