/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/



/**************************************************************************
 File:  solvers.h

 Purpose:  Contains global definitions for the solvers

 Created By:  J. E. Tvedt
 **************************************************************************/

#ifndef __SOLVERS_H_
#define __SOLVERS_H_
#include "StateLevelObjects/BaseStateLevel.h"
#include "StateLevelSolvers/common/BaseStateLevelSolver.h"
#include "BaseClasses/MobiusEnums.h"
#include <stdio.h>

/* types of solvers */
#define SOLVER_TRS  0
#define SOLVER_ISS  1
#define SOLVER_DSS  2
#define SOLVER_DISS 3
#define SOLVER_ADISS 4
#define SOLVER_ATS  5
#define SOLVER_ARS  6
/* A VERY SMALL NUMBER */
#define VERY_SMALL 1.0e-11

/* A VERY LARGE NUMBER */
#define VERY_LARGE 1.0e+9

/* COMPUTES THE MAXIMUM OF TWO VALUES */
#define maxi(u,v)(((u)>(v))?(u):(v))

/* COMPUTES THE MINIMUM OF TWO VALUES */
#define mini(u,v)(((u)<(v))?(u):(v))

#ifndef IEEE
#define IEEE  		1.7E+300
#endif

class ArgvOrder {
public:
	ArgvOrder(int argc, char** argv);
	~ArgvOrder();
	void Save(int argc, char** argv);
	void Restore();

private:
	char** copy_argv;
	char** main_argv;
	int main_argc;
};

/* ALPHABETICAL LISTING OF SOLVER FUNCTIONS */
void annotate(const char *name, FILE *to);
double comp_eps();
void comp_plot_cdf(BaseStateLevel* model, BaseStateLevelSolver* solver, int pv_index, char *plot_filename, RateType *prob, RateType *reward, RateType timept,
		double mean, double variance, FILE *output);

void dsort(unsigned long n, double ra[], double rb[]);
void fatal_error(FILE *fp, char *string);
void FindNonzeroRewards(BaseStateLevel* model, int& nonzero_rewards, int*& reward_mapping);
void FindNonzeroRewards(BaseStateLevel* model, int& nonzero_rewards, int*& reward_mapping, bool& has_impulse_reward, bool*& has_reward);
char* getAsciiTime();

void writeSplotFile(char *plot_filename, char *title, char *prob_formula, BaseStateLevelSolver *solver, char *pvname, double timept, double mean,
		double variance, int num_pdf, double *pdf_x, double *pdf_y, FILE*output);

void perfcdb(BaseStateLevel* model, BaseStateLevelSolver* solver, double *prob, FILE *fout, FILE *fcsv, int time_mode, int time_index, double starttime,
		double stoptime, int num_times, bool comp_flag, RewardType validRewardType, char *db_entity, int db_exp_index);
void plot_cdf(BaseStateLevel* model, BaseStateLevelSolver* solver, int pv_index, char *plot_filename, RateType *prob, /* Array of state occupancy probabilities */
RateType *reward, RateType timept, /* Time point of interest */
double mean, double variance, FILE *output);

void gen_pdf_db(BaseStateLevel* model, BaseStateLevelSolver* solver, int pv_index, char *plot_filename, RateType *prob, RateType *reward, int time_mode,
		int time_index, RateType starttime, RateType stoptime, double mean, double variance, char *db_entity, int exp_index, FILE *output, FILE*csvfile);
//the general function
void pois_pts(bool acc_poisson_prob, double lambda, double epsilon, unsigned int &L, unsigned int &R, double *&pois_coef, unsigned int minimum_R, FILE *fout);
//for trs solver
void pois_pts(double lambda, double epsilon, unsigned int &L, unsigned int &R, double *&pois_coef, FILE *fout);
//for ars solver
void pois_pts(bool acc_poisson_prob, double lambda, double epsilon, // truncation epsilon
		unsigned int &L, unsigned int &R, double *&pois_coef, FILE *fout);
//for ats solver
void pois_pts(double lambda, double epsilon, unsigned int &L, unsigned int &R, double *&pois_coef, unsigned long minimum_R, FILE *fout);

double port_pow(double base, int power);
void solver_banner(char *prog, /* Name of program (e.g. ars) */
char *title, // Title of program (e.g. "ACCUMULATED REWARD SOLVER")
		char *contents, // Contents of file (e.g. "RESULTS" or "DEBUG INFO")
		char *project_name, /* Name of project */
		char *study_name, /* Name of study */
		char *ssg_full_path, char *experiment_name, /* Name of experiment */
		const char *comments_filename, /* Name of comments file */
		FILE *to); /* Pointer to output file */
char* strDup(const char* in);
void writeCSVFile(FILE *CSVFILE, BaseStateLevelSolver *solver, int exp_index, int pv_index, char *pvname, double starttime, double stoptime, double mean,
		double variance, int numpdf, double *pdf_x, double *pdf_y);
void writeCSVFile(FILE *CSVFILE, BaseStateLevelSolver *solver, int exp_index, int pv_index, char *pvname, double starttime, double stoptime,
		char *resultsLabel, char *label1, double value1, char *label2, double value2, int numpdf, double *pdf_x, double *pdf_y);

#endif
