/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#include <iostream>
#include <math.h>
#include <vector>
#include <stdio.h>
#include "StateLevelObjects/BaseStateLevel.h"
#include "StateLevelSolvers/common/BaseStateLevelSolver.h"
#include "StateLevelSolvers/common/solvers.h"
#include "StateLevelSolvers/common/common_db.h"

struct Matrix {
	RateType *row_element;
	SizeType *col_index;
	int length;
};

class DSSSolver: public BaseStateLevelSolver {
public:
	typedef vector<RateType> VectorType;

	DSSSolver(BaseStateLevel *_model, int argc, char** argv);
	~DSSSolver();
	void solve();
	void initialize();

	RateType getSolutionVectorElement(SizeType s);
	void getSolutionVectorAsArray(RateType*& vec);
protected:
	virtual void ShowCmdLineOptions();
	void print_states();

private:

	void DisplayBanners();
	void terminate();
	void output(double lambda, int nonzero_rewards, unsigned int* l_trunc_points, unsigned int* r_trunc_points, double* trunc_error);
	void updatedb();
	void initializeSolver();
	void readMatrix(struct Matrix* mat);
	void performLUDecomp();
	void pivotMatrix(unsigned long k);
	void calcRowCost(unsigned long k);
	void calcColCost(unsigned long k);
	void swapCols(int row, int col1, int col2);
	void swapRows(int row1, int row2);
	int setVal(struct Matrix * mat, int i, int j, RateType val);
	RateType getVal(int i, int j);
	RateType upperTriangular(int i, int j);
	void lowerTriangular(int i, int j, RateType beta);
	void forwardSub(RateType* matY, RateType* matB, struct Matrix* matA);
	void backwardSub(RateType* matY, RateType* matZ, struct Matrix* matA);
	void findSolution(RateType* z, RateType* x);
	void iterativeRefine(struct Matrix* matA, RateType* sol, RateType* newSol);
	RateType norm(RateType* mat);
	void colPos(int j);

	// Options/Parameters for DSS solver
	RateType epsilon;
	RateType tolerance;
	RateType tol;
	bool comp_flag;
	short int stability;
	unsigned long int rows;

	// Local variables to DSS implementation
	RateType* newX;
	unsigned int numNonzeroElements;
	unsigned long newPivots;
	unsigned long* colCost;
	unsigned long* rowCost;
	unsigned long* colSwap;
	unsigned long* rowSwap;
	int * links;
	RateType* b;
	RateType minRate;
	int working_debug_flag;

	SizeType n; // Number of states
	struct Matrix* A;

};

