/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: ISSSolver.h
 *
 *      Authors: Salem Derisavi, William H. Sanders
 *      Creation Date: 2003
 *
 *      Copyright (c) 1994-2004 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

/***********************************************************************
 * A base class for the SOR and Jaboci iterative steady state
 * solvers. The only difference between SOR and Jacobi is that SOR
 * uses column oriented access while Jacobi solver doesn't require any
 * order on the set of transitions. Therefore, SOR and Jacobi use the
 * Column and AllEdges containers of a state-level AFI object
 * respectively
 ************************************************************************/

#ifndef __ISS_SOLVER_H_
#define __ISS_SOLVER_H_

#include <stdio.h>
#include "StateLevelObjects/BaseStateLevel.h"
#include "StateLevelSolvers/common/solvers.h"
#include "StateLevelSolvers/common/common_db.h"
#include "StateLevelSolvers/common/BaseStateLevelSolver.h"

class ISSSolver: public BaseStateLevelSolver {
public:
	ISSSolver(BaseStateLevel* _model, int argc, char** argv, const char* _solver_name, const char* _solver_banner_title);

	~ISSSolver();
	void initialize();
	void solve();

	RateType getSolutionVectorElement(SizeType s);
	void getSolutionVectorAsArray(RateType*& vec);
protected:

	RateType vnorm();
	virtual void compute_diagonal() = 0;
	virtual void perform_iteration() = 0;
	virtual void initialize_ds() = 0;
	virtual void terminate_ds() = 0;
	void output();
	void updatedb();
	void DisplayBanners();
	virtual void ShowCmdLineOptions();
	void print_states();
	void cleanup();

	RateType epsilon;
	RateType omega; //The acceleration factor
	int max_iterations;
	bool fixup;
	bool comp_flag;

	char* solver_banner_title;

	RateType* diag; /* Diagonal Values */
	RateType* prob_vec;

	//Number of nonzero elements of the Q matrix.
	//That is equal to the number of transition plus the number of
	//states (for diagonal values of the Q matrix)
	unsigned int num_nonzero_elements;
	int num_of_iterations;
	RateType max_diff;
	RateType overflow;
	DBbool did_converge;

	BaseStateLevel::EdgeBaseIterator* it;
	BaseStateLevel::EdgeBaseContainer* container;
	EdgeBuffer t;

	SizeType n;
	RateType inv_omega;
};

#endif
