/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __TAKAHASHI_SOLVER_H__
#define __TAKAHASHI_SOLVER_H__

#include "StateLevelSolvers/common/BaseStateLevelSolver.h"
#include <vector>

class TakahashiSolver: public BaseStateLevelSolver {
public:
	typedef vector<RateType> VectorType;

	TakahashiSolver(BaseStateLevel *_mode, int argc, char** argv);
	~TakahashiSolver() {
	}
	void initialize();
	void solve();

protected:
	virtual void ShowCmdLineOptions();

private:
	void DisplayBanners();
	void DisplayBanners(FILE* file);
	void terminate();
	void output();
	void updatedb();
	double normalize_subvector(VectorType& x, SizeType start, SizeType end, VectorType& out, SizeType start_out);
	void compute_diagonal();
	double compute_residual(int k);
	void partition_transition_matrix();
	void ConstructAndSolveAggregationMatrix();
	void ConstructAggregationMatrix();
	void SolveAggregationMatrix();
	void deallocate_reserved_spaces();

private:
	RateType outer_epsilon;
	int max_outer_iterations;
	int num_of_outer_iterations;
	int x_parts;
	int y_parts;
	VectorType residual;
	Partition row, column;
	VectorType diag;
	RateType max_residual;

	//variables specific to the solution of the aggregation matrix
	RateType** aggregation_mat;
	int AggregationMaxIterations;
	RateType AggregationOmega;
	RateType AggregationEpsilon;

	//variables specific to the disaggregation phase
	int DisaggregationMaxIterations;
	RateType DisaggregationOmega;
	RateType DisaggregationEpsilon;

	//An array of booleans used to represent a 2D array of size
	//row.max*column.max of boolean. An element i,j shows whether
	//block i,j is empty, i.e., does not have any non-zero element
	bool *submatrix_empty;

	BaseStateLevel::EdgeBaseIterator* SM_it;
	BaseStateLevel::EdgeBaseContainer* SM;
	EdgeBuffer t;

	VectorType non_homogen;
	RateType* xi;
	VectorType phi;
	SizeType col_length, col_start, col_end;

};

#endif
