/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#include <iostream>
#include <math.h>
#include <vector>
#include <stdio.h>
#include "StateLevelObjects/BaseStateLevel.h"
#include "StateLevelSolvers/common/BaseStateLevelSolver.h"
#include "StateLevelSolvers/common/solvers.h"
#include "StateLevelSolvers/common/common_db.h"

class TRSSolver: public BaseStateLevelSolver {
public:
	typedef vector<RateType> VectorType;

	TRSSolver(BaseStateLevel *_model, int argc, char** argv);
	~TRSSolver();
	void initialize();
	void solve();

	RateType getSolutionVectorElement(int time_point, SizeType s);
	void getSolutionVectorAsArray(int time_point, RateType*& vec);
protected:
	virtual void ShowCmdLineOptions();
	void print_states();
	void getUnionTimes(BaseStateLevel *model, vector<double> &times);

private:
	void DisplayBanners();
	void terminate();
	void output(double lambda, int nonzero_rewards, unsigned int* l_trunc_points, unsigned int* r_trunc_points, double* trunc_error);
	void updatedb();

	RateType epsilon;
	int no_of_time_points;
	RateType* time_points; //The time points in which to compute the prob. dist.
	bool comp_flag;
	bool UsePVTypes;

	RateType* diag; /* Diagonal Values */
	RateType* prob_vec; /* current DTMC probability vector */
	RateType* prob_vec_2; /* previous DTMC probablity vector */
	RateType** solutions; /* array of solution vectors; a vector for each time point*/

	RateType *lambda_t; /* multiplication of lambda and each time point */
	RateType *trunc_error; /* Truncation error for each time point */
	RateType **poisson_coef;
	unsigned int *l_trunc_points, *r_trunc_points;
};
