/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File:  ResultsDB.hpp
 *
 *      Authors:  Tod Courtney, William H. Sanders
 *      Creation Date: 2006
 *
 *      Copyright (c) 1999-2000 The Board of Trustees of the University of Illinois
 *                         All rights reserved.
 *
 \*****************************************************************************/

#ifndef RESULTSDB_H_
#define RESULTSDB_H_

#define SHOW_DB_INFO 1

#if PQINSTALLED
#include "libpq-fe.h"
#endif

#include <iostream>
#include <stdio.h>
#include <string.h>

class BaseStudyClass;

/**
 *  ResultsDB -- This class contains database instrumentation methods
 *              used in C++ solvers
 *
 */
class ResultsDB {
private:
	char Host[100];
	int Port;
	char Entity[100];
	char ExpName[100];
	char User[100];
	char Password[100];
#if PQINSTALLED
	PGconn *DBconn;
#endif
	char _qstr[512];

public:
	/**
	 *
	 */
	ResultsDB(const char* host = nullptr, int port = -1,
				const char* user = nullptr, const char* password = nullptr,
				const char *entity = nullptr);

	/**
	 *
	 */
	~ResultsDB();

	/**
	 *  initialize the database
	 */
	bool init(const char* host = nullptr, int port = -1,
				const char* user = nullptr, const char* password = nullptr,
				const char* entity = nullptr);

#if PQINSTALLED
	PGresult * query(char *query);
#endif

	int getRunId(char *runname);

	/**
	 *  helper function that sets the db parameters, called by both init
	 *  and the constructor. only updates values if the new value is not
	 *  NULL (or -1 for ints)
	 */
	void setDBInfo(const char* host, int port, const char* user, const char* password, const char* entity);

	bool writeSSGExperInfo(char * TheExpName, unsigned long TheNumStates);

	bool writeGVValues(BaseStudyClass *TheStudy, const char *TheExpName);

	bool writeSingleGV(char *TheGVName, char *TheSubarray, char *TheExpName, void * TheValue);

	void overrideGVs(char * TheOverrideString, BaseStudyClass *TheStudy);
}; // end BaseStudyClass

#endif // RESULTSDB_H_
