/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: FileType.h
 *
 *      Authors: John Sowder, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

// this is the header file for the FileTypes class
#ifndef FILETYPE_H
#define FILETYPE_H

#include <fstream>
#include <stdio.h>
#include <string.h>
#include <cstdlib>

/** John Sowder
 * Version 1.0
 * 8-5-98
 * This class implements the functionality of multiple file formats
 * within the Mobius state space genertor as described in State Space
 * Generation techniques in the Mobius modeling framework
 */

using namespace std;

class FileType {

private:
	/** Most the the data members listed here are temporary variables as
	 the names suggest and are used when reading in the temporary files
	 */
	unsigned long temp_num_states, temp_state_num, total_size;
	long temp_long;
	double temp_double, *next_rate_array;
	unsigned long *state_num_array, *next_state_array, tempzero;
	short temp_filetype;
	unsigned long temp_current_state, temp_next_state, index;

public:
	/** Base Constructor
	 */
	FileType() {
	}
	;
	/** constructor that sets the filetype
	 * @parem short filetype
	 */
	FileType(short) {
	}
	;
	/** The Desctuctor
	 */
	~FileType() {
	}
	;
	/** This method takes the temp file written dynamically by the ssg
	 * and rewrites a new file according to the type of format specified
	 * by the Mobius ssg editor
	 *
	 * @parem short filetype  The file type specified by the user
	 * @parem char *TheTempFileName The name of the temporary file
	 *              containing the state-transition rate matrix
	 * @parem char *TheNewFileName  The name of the file of where to store
	 *               the converted state transition-rate matrix
	 *
	 */
	void FileFix(short, char*, char*);
	/** This method takes the tempfile and converts the format from row to column
	 * and returns in the pointers to the results
	 *
	 * @parem unsigned long *addy_arrray An array of pointers to the column
	 *                       format conversion result
	 * @parem short keep_self  A flag used in determined whether or not to
	 *                         keep the self transitions 			   * @parem char *TheTempFileName  The name of the temp file containing the
	 *                               state-transition rate matrix in row format
	 */

	void MakeColumnFormat(short, char*);
};

#endif
