/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: HybridHash.h
 *
 *      Authors: John Sowder, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

// This is the header file for the Hybrid Hash class

#ifndef HASH_H
#define HASH_H
#include "../Performance_Variables/PVModel.hpp"
#include <iostream>

/** HybridHash.h
 * John Sowder
 * Version 1.0
 * 8-5-98
 * This class implements the hash function and hash table data structure
 * define in State Space Generation Tehhniques in the Mobius Modelling
 * Framework.  The Class mainly uses two methods lookup and insert.  The
 * getHashKey method is the implmentation of the hash function
 */

static long PrimeArray[22] = { 131071, 393209, 786419, 1179637, 1966049, 2752481, 4325341, 6684607, 9437101, 15335303, 21000169, 31000303, 51000353, 71000383,
		0 };// supports up to 21M states at 0.3
// 35.5M with 0.5
// if more are needed add more prime number
// values to the end of this list

class HybridHash {

private:
	/** HashTable is the hash table data structure
	 */
	unsigned int *HashTable;
	/**  i,j,k are counters
	 */
	unsigned int i, j, k;
	/**  NumRehases is the current number of rehashes during execution
	 *  tempsum is the temp var used to hold the hash key
	 *  numcoll is the number of collisions per lookup used in collision policy
	 */
	unsigned int NumRehashes, tempsum, numcoll;
	PVModel *The_Model;
	/** The structure of arrays
	 */
	char ***ArrayStruct;
	/** These parameters are set in the constructor
	 */
	unsigned long State_Size, current_array, ArraySize, Hash_Size, Dist;
	/** These parameters are also set in the constructor
	 */
	unsigned long NumArrays, CurrentSize;
	/** This is the method that adds arrays to the array structures as
	 * needed based on the current number of states in the model
	 */
	void AddArray();
	/** This method does the rehashing, using the PrimeValues array shown above
	 */
	void Rehash();

	/** The value used to decide when to rehash the hash function
	 */
	double HashValue;

public:
	/** Base constructor
	 */
	HybridHash() {
	}
	;
	/** constructor that sets all the hash specific parameters needed
	 * during execution
	 */
	HybridHash(PVModel*, unsigned long, unsigned long, unsigned long, unsigned long, unsigned long, double);
	/** the destructor
	 */
	~HybridHash();
	// gcla ~HybridHash() {};
	/** This method deletes the hash table
	 */
	void DeleteMem() {
		delete[] HashTable;
	}
	/** This method inserts the state into the hash table and its associated
	 * data structures
	 * @parem char *State The state to be inserted
	 * @parem unsigned long The current state number
	 */
	void Insert(char*, unsigned long);
	/** This method checks to see if the state is in the hash table
	 * @parem char *State The state to do the lookup on
	 */
	long Lookup(char *);
	/** This method uses the hash function to compute the hash key
	 * char *TheState The State to use to hash on
	 * long TheSize The current hash table size
	 */
	inline unsigned int getHashKey(char *TheState, long);

};

#endif
