/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: gen.h
 *
 *      Authors: John Sowder, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef GEN_H
#define GEN_H

#include <iostream>
#include <fstream>
#include <iomanip>
#include <math.h>
#include <string>
#include <stdio.h>
#include <unistd.h>
#include <streambuf>
#include "../Performance_Variables/PVModel.hpp"
#include "../Study/BaseStudyClass.hpp"
#include "../BaseClasses/BaseGroupClass.h"
#include "../BaseClasses/StandardErrors.h"

#define max_no_states 1000

/**
 *  I believe that this data structure
 *  holds a model's state
 *  and the unique state number used to
 *  identify the state to the state space
 *  generator.  Objects of this type are
 *  added to a Queue object. (JMD)
 *
 *  @author John Sowder
 *  @version 1.0
 */
class state_struct {

public:

	/**
	 *  A pointer to a saved value of
	 *  a model's state (?)
	 */
	char *state_ptr;

	/**
	 *  The unique state number issued by
	 *  the state space generator
	 */
	long int num;

	/**
	 *  Default constructor
	 */
	state_struct() {
	}
	;

	/**
	 *  Overloaded constructor that creates
	 *  a state container of the appropriate
	 *  size.
	 *
	 *  @param TheStateSize The size of the
	 *         state container.
	 */
	state_struct(unsigned long TheStateSize) {
		if ((state_ptr = new char[TheStateSize]) == NULL) {
			MemoryAllocationError();
		} // end if
	} // end state_struct::state_struct(unsigned long)

	/**
	 *  Destructor
	 */
	~state_struct() {
		// gcla - no need for this check in C++
		//if(state_ptr != NULL){
		delete[] state_ptr;
		//} // end if
	} // end state_struct::~state_struct

}; // end state_struct class


#endif

