/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File: queue.h
 *
 *      Authors: John Sowder, Daniel D. Deavours, William H. Sanders
 *      Creation Date: 1999
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

// Queue header file

#ifndef QUEUE_H
#define QUEUE_H
#include <iostream>
#include "gen.h"

/**
 *  Default size of the queue(?)
 */
static const int Default_Size = 5000;
//static const int Default_Size = 100;

/**
 *  This class implements a basic queue with dynamic resizing.
 *
 *  @author John Sowder
 *  @version 1.0
 */
class Queue {

public:

	/**
	 *  Default constructor
	 */
	Queue() {
	}
	; // end Queue::Queue()

	/**
	 *  Queue constructor
	 *
	 *  @parem InitialSize The initial size of the queue
	 */
	Queue(unsigned long InitialSize);

	/**
	 *  Destructor
	 */
	virtual ~Queue();

	/**
	 *  Delete the Queue method
	 */
	inline void DeleteMem() {
		delete[] Q_Array;
	} // end Queue::DeleteMem();


	/**
	 * Stores a new state in the end of the queue
	 */
	void Enqueue(state_struct * X);

	/**
	 * Removes the state from the head of the queue
	 */
	state_struct *Dequeue();

	/**
	 *  This method makes the queue empty without
	 *  actually deleting anything.
	 */
	void Make_Empty();

	/**
	 *  This method makes the queue larger and
	 *  enqueues the state_struct passed in as
	 *  an argument.
	 *
	 *  @param NextItem A item to be added to
	 *         the queue after the queue
	 *         has been enlarged.
	 */
	void MakeLarger(state_struct* NextItem);

	/**
	 *  This method determines if the queue
	 *  is currently empty.
	 *
	 *  @return True if the queue is empty, else
	 *          false
	 */
	inline bool Is_Empty() {
		if (Q_Size == 0) {
			return true;
		} // end if
		else {
			return false;
		}// end else
	} // end Queue::Is_Empty()

	/**
	 *  This method determines is the
	 *  queue is currently full.
	 */
	inline bool Is_Full() {
		if (Q_Size == Full_Queue) {
			return true;
		} // end if
		else {
			return false;
		} // end else
	} // end Queue::Is_Full()

	/**
	 *  This method prints all values in
	 *  the queue, used for debugging
	 */
	void Print();

private:

	/**
	 * This full queue size
	 */
	unsigned int Full_Queue;

	/**
	 * The State Size of the model
	 */
	unsigned long State_Size;

	/**
	 * Front of the queue pointer
	 */
	int Q_Front;

	/**
	 * Rear of the queue pointer
	 */
	int Q_Rear;

	/**
	 * The queue size
	 */
	int Q_Size;

	/**
	 * counter
	 */
	long int i;

	/**
	 * current number of reallocations
	 */
	short num_realloc;

	/**
	 *
	 */
	state_struct **Q_Array; // pointer array

	/**
	 *  The increments the head of the queue
	 */
	void Increment(int &X);

}; // end Queue class

#endif

