/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef GRAPH_SYM_DS_H
#define GRAPH_SYM_DS_H

#include "BaseClasses/BaseModelClass.h"
#include "GraphAtomicModel.h"
#include <vector>
#include <map>
#include "BaseClasses/symbolic/MappedMDD.h"
#include "BaseClasses/symmetry/SymmetryDetection.h"

class GraphSymDS {
public:
	/*******************/
	/** Public methods */
	/*******************/
	GraphSymDS(BaseModelClass* RootPVModel);
	~GraphSymDS();

	int getNumLevels();
	int getNumAtomicModels();

	/** Get the number of depencies. */
	int GetNumDependencies(GraphAtomicModel *gam);

	/** Get Smallest level */
	int GetSmallestDepLevel(GraphAtomicModel *gam);

	/** Get Largest level */
	int GetLargestDepLevel(GraphAtomicModel *gam);

	/** Sets the SS order in SymmetryDetection class */
	void FindSSOrder();

	/** Used in trimming the lumped MDD */
	bool IsPartiallyCanonical(int level, int * indices);
	std::vector<State*> * StateSpace;
	byte* af_state;
	int TotalStateSize;
	int NumLevels;

	/**********************/
	/** Public Attribute  */
	/**********************/
	std::vector<GraphAtomicModel*> AtomicModels;
	std::vector<BaseModelClass*> AllModels;

	/** One for each Atomic Model */
	int NumNonLocalMatrices;

	/** One for each Atomic Model containing state */
	int NumLocalMatrices;

	/** One for each model containing state */
	int NumLocalStateSpaces;

	/** Maps Model pointers to MDD levels */
	std::map<BaseModelClass*, int> Model2MDDLevel;

	/** \todo Descriptions */

	/** Map from Submodel index to Local Matrix index */
	std::vector<int> SubmodelIndex2LMIndex;

	/** Map from Submodel index to Non Local Matrix index */
	std::vector<int> SubmodelIndex2NLMIndex;

	/** Map from MDD level to State Space index */
	std::vector<int> MDDLevel2SSIndex;

	/** \todo Eh? */
	TOrder *order;

	/** Symmetry Detection class */
	SymmetryDetection *SymDet;
	int SetStateSizes();
	int GetPartitionSize(int i);

private:
	/********************/
	/** Private methods */
	/********************/

	/** Initalize the Atomic Models */
	void InitializeAtomicModels(BaseModelClass * model, int &level);

	/** Initializes other vectors (maps, etc) */
	void InitializeOtherVectors(BaseModelClass* model, int & LMindex, int &NLMindex, int & SSindex);

	/** Searches for model in all of the Graph Atomic Models */
	bool SearchAtomicModels(BaseModelClass* model);

	/** Initializes the dependency lists using
	 * the Global Depency List created in
	 * InitializeOtherVectors() method.
	 */
	void InitializeDependencyLists();

	/** Initializes the levels of the dependency
	 * lists for each Atomic Model.
	 */
	void InitializeDependencyLevels();

	/*************************/
	/** Private Data members */
	/*************************/

	/** Root Composed Model */
	BaseModelClass* root;

	/** Global vector to help create Dependency Lists.
	 *
	 * This is similar to the 'ancestors' structures in sgen.
	 * It may have duplicates, which are trimmed in InitializeDepencyLists()
	 */
	std::vector<std::pair<BaseModelClass*, BaseModelClass*> > GlobalDepList;
};

#endif
