/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __LUMPING_MXD__
#define __LUMPING_MXD__

#include "BaseClasses/symbolic/lumping-tarjan.h"
#include "BaseClasses/symbolic/MatrixElementList.h"
#include "PVMTMDDCreator.h"
#include "BaseClasses/symbolic/md.h"
#include <vector>

typedef Lumping<MTMDDNode*, MatrixElementList> TLumpingMxD;

class LumpingMxD {
public:
	typedef TLumpingMxD::TPartition TPartition;
	typedef TLumpingMxD::TChain TChain;
	typedef TLumpingMxD::successor successor;
	typedef TLumpingMxD::predecessor predecessor;
	LumpingMxD(MD* _md, MDNode* _mdroot, vector<int>& _SSSizes, vector<int>& _MDDLevel2SSIndex, PVMTMDDCreator* _UnlumpedPVMTMDD);
	~LumpingMxD();
	void LumpMxD();
	TPartition& getPartition(int level) {
		return Partitions[level];
	}
	MDNode* getLumpedMxD() {
		return lumped_mdroot;
	}
private:
	void LumpMxDLevel(int level);
	void ComputeInitialPartition(TPartition& Q, int level, int SSSize);
	MDNode* BuildLumpedMxD(MDNode* node, int level);
	void ReadChainsFromMxDLevel(int level, TChain*& chains);
	void ReadChainFromMxDNode(MDNode* mdnode, TChain& chain);
	bool IsPartitionCorrect(TChain* chains, int no_of_chains, TPartition& Q);
	MDNode* ComputeLumpedMxDNode(int level, TChain& chain, TPartition& Q);
	void ClearLumpedMxDNodes();
	void ConstructMDNodesLists(MDNode* root, int level = 0);
	void MD_DFS(MDNode* mdnode, set<MDNode*>& s);
	void MD_DFS_UpToLevel(MDNode* mdnode, int level, set<MDNode*>& s);

	MD* md;
	MDNode* mdroot;
	MDNode* lumped_mdroot;
	TPartition* Partitions;
	vector<int> SSSizes;
	vector<int> MDDLevel2SSIndex;
	vector<MDNode*>* MDnode_lists;
	MTMDDNode::node_slist_type** PVMTMDDnode_lists;
	PVMTMDDCreator* UnlumpedPVMTMDD;
	typedef map<MDNode*, MDNode*> LumpingMap;
	LumpingMap LumpedMxDNodes;
	map<MDNode*, MDNode*> GlobalLumpedMxDNodes;
    
#if (HAS_FORWARD_LIST)
    typedef std::forward_list<TPartition::iterator> block_list_t;
    typedef std::forward_list<TLumpingMxD::successor> successor_list_t;
#else
	typedef slist<TPartition::iterator> block_list_t;
    typedef slist<TLumpingMxD::successor> successor_list_t;
#endif

};

typedef pair<int, TLumpingMxD::TPartition::iterator> TPair;
struct less_first_component {
	bool operator()(const TPair& x, const TPair& y) {
		return x.first < y.first;
	}
};

#endif
