/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __SymDT_H_
#define __SymDT_H_

#include "BaseClasses/BaseModelClass.h"
#include <vector>
#include <map>
#include "BaseClasses/symbolic/MappedMDD.h"

class Ancestor;

#if (HAS_FORWARD_LIST)
typedef std::forward_list<Ancestor> ancestor_list_t;
#else
typedef slist<Ancestor> ancestor_list_t;
#endif


class Ancestor {
public:
	Ancestor(int _level, BaseModelClass* _model) {
		level = _level;
		model = _model;
	}
	int level;
	BaseModelClass* model;
};

class AtomicModel {
public:
	AtomicModel(BaseModelClass* _model, int _no_of_ancestors) {
		model = _model;
		no_of_ancestors = _no_of_ancestors;
		ancestors_levels = new int[no_of_ancestors];
		ancestors = new BaseModelClass*[no_of_ancestors];
	}
	~AtomicModel() {
		delete[] ancestors_levels;
		delete[] ancestors;
	}
	BaseModelClass* model;
	int no_of_ancestors;
	int* ancestors_levels;
	base_action_list_t exp_local_actions;
	base_action_list_t imm_local_actions;
	base_action_list_t global_actions;
	BaseModelClass** ancestors;
};

#define SYMSSG_ERR_MODEL_IS_EMPTY 1
#define SYMSSG_ERR_GLOBAL_NONEXP_ACTIONS 2
#define SYMSSG_ERR_UNSUPPORTED_DISTRIBUTION 3
#define	SYMSSG_ERR_VANISHING_INIT_STATE 4

class SymDS {
public:
	SymDS(BaseModelClass* RootModel);
	~SymDS();
public:
	void print();
	int ComplyWithRestrictions();

	int no_of_atomic_models() {
		return atomic_models.size();
	}
	int no_of_levels() {
		return models.size();
	}
	vector<AtomicModel*> atomic_models;
	vector<BaseModelClass*> models;
	vector<int> SubmodelIndex2LMIndex;
	vector<int> SubmodelIndex2NLMIndex;
	vector<int> MDDLevel2SSIndex;
	TOrder *order;
	int no_of_local_matrices;
	int no_of_nonlocal_matrices;
	int no_of_state_spaces;

private:
	void InitializeDataStructure();
	void InitializeOtherVectors(BaseModelClass* model, int& LMindex, int& NLMindex, int& SSindex);
	void InitializeAncestorLevels();
	void InitializeAtomicModels(BaseModelClass* const model, ancestor_list_t& ancestors_list, int& level);
	void InitializeTOrderDS();
	void InitializeTOrderDS(BaseModelClass* model, int& rep_index, int parent_rep_index, int& no_of_levels, int& level);
	BaseModelClass* root;
	map<BaseModelClass*, int> Model2MDDLevel;
	int no_of_rep_models;
};

#endif
