/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      Copyright (c) 1994-2008 William H. Sanders, and
 *      The Board of Trustees of the University of Illinois
 *      All rights reserved.
 *
 \*****************************************************************************/

#ifndef __MD_H_
#define __MD_H_

#include "cache.h"
#include "mdnode.h"
#include "mddnode.h"
#include "ssnode.h"
#include <vector>
#include <stdio.h>
#include <map>

class MD {
public:
	typedef MDNode::DIM_TYPE DIM_TYPE;
	MD(int _UniqueTableSize, int _SubmatCacheSize, int _AddCacheSize, MDDNode* _MDDZero, MDDNode* _MDDOne);
	~MD();
	MDNode* Submat(short level, MDNode* A, SSNode* Major, SSNode* Minor);
	MDNode* Add(short level, MDNode* A, MDNode* B);
	void AddElement(short level, TElement* result, TElement* elem_A);
	void CopyElement(MDNode* C, MDNode* source, DIM_TYPE source_major, DIM_TYPE source_minor, int& source_idx);
	void addNode(MDNode* node);
	void deleteNode(MDNode* node) {
		node->decRef();
	}
	MDNode* insertNode(MDNode* node);
	MDNode* insertNodeUnique(MDNode* node);
	void removeNode(MDNode* node);
	pair<MDNode*, bool> findOrAddNode(MDNode* node);
	MDNode* readTree(FILE *f);
	void print();
	void GC();
	void EnableGC() {
		GCEnabled = true;
	}
	void DisableGC() {
		GCEnabled = false;
	}
	void report();
	int getNoOfNodes() {
		return UniqueTable.getNoOfNodes();
	}
	void resize(int _UniqueTableSize, int _SubmatCacheSize) {
		UniqueTable.resize(_UniqueTableSize);
		SubmatCache.resize(_SubmatCacheSize);
	}
	void registerCache(ComputedCacheBase* Cache) {
		UniqueTable.registerCache(Cache);
	}

	typedef NodeUniqueTable<MDNode> MDNodeUniqueTable;

	MDNodeUniqueTable::HashTableType::iterator begin() {
		return UniqueTable.begin();
	}
	MDNodeUniqueTable::HashTableType::iterator end() {
		return UniqueTable.end();
	}
private:
	inline bool ProduceOneElement(short level, TElement* element, MDNode* B, MDDNode* Major, MDDNode* Minor, DIM_TYPE major, DIM_TYPE minor, DIM_TYPE b_major,
			DIM_TYPE b_minor);
	MDDNode* MDDZero;
	MDDNode* MDDOne;

	class SubmatCacheKey {
	public:
		SubmatCacheKey() {
			reset();
		}
		SubmatCacheKey(short _level, MDNode* _A, MDDNode* _Major, MDDNode* _Minor) {
			level = _level;
			A = _A;
			Major = _Major;
			Minor = _Minor;
		}

		//Computes the hash function on the key
		size_t operator()(SubmatCacheKey& key) {
			return (int) level * 103 + (unsigned int) key.A->getID() * 97 + key.Minor->getID() * 199 + key.Major->getID() * 149;
		}

		//Compares two keys
		bool operator==(SubmatCacheKey& key2) {
			return (level == key2.level && (void*) A == (void*) key2.A && (void*) Minor == (void*) key2.Minor && (void*) Major == (void*) key2.Major);
		}
		void reset() {
			Minor = Major = NULL;
			A = NULL;
			level = -1;
		}

		//A cache entry is dead if the MDD node of the key is dead
		bool isDead() {
			return A->isDead() || Major->isDead() || Minor->isDead();
		}
	private:
		short level;
		MDNode* A;
		MDDNode* Major;
		MDDNode* Minor;
	};

	class CSubmatCache: public ComputedCache<SubmatCacheKey, MDNode> {
		void GC();
	} SubmatCache;

	class AddCacheKey {
	public:
		AddCacheKey() {
			reset();
		}
		AddCacheKey(short _level, MDNode* _AA, MDNode* _BB) {
			level = _level;
			A = _AA;
			B = _BB;
		}

		//Computes the hash function on the key
		size_t operator()(AddCacheKey& key) {
			return (int) level * 103 + (unsigned int) key.A->getID() * 97 + key.B->getID() * 199;
		}

		//Compares two keys
		bool operator==(AddCacheKey& key2) {
			return (level == key2.level && (void*) A == (void*) key2.A && (void*) B == (void*) key2.B);
		}
		void reset() {
			A = B = NULL;
			level = -1;
		}

		//A cache entry is dead if the MDD node of the key is dead
		bool isDead() {
			return A->isDead() || B->isDead();
		}
	private:
		short level;
		MDNode* A;
		MDNode* B;
	};

	class CAddCache: public ComputedCache<AddCacheKey, MDNode> {
		void GC();
	} AddCache;

	MDNodeUniqueTable UniqueTable;

	//The maximum allowable load factor(number of elements divided by
	//the number of buckets) for the unique table. When this load factor
	//is exceeded after adding a new node, the hashtable size is
	//increased by 1. (The elements in the hashtable will not
	//redistributed)
	int MAXLOADFACTOR;

	//The maximum allowable ratio of number of dead nodes and number of
	//all nodes ( < 1). Garbage collection is ignited when this ratio is
	//violated.
	float MAXDEAD;

	typedef map<int, MDNode*> mapping;
	MDNode* dfsReadTree(FILE* f, mapping& nodetable);
	bool GCEnabled;
};

#endif

