
#include "Study/AvNumStudy/AvNumStudyRangeStudy.h"

//******************************************************
//Global Variables
//******************************************************
short ACCMINTOASSEMBLE;
short ACCSIZE;
short BINSIZE;
double RecoverRate;

//********************************************************
//AvNumStudyRangeStudy Constructor
//********************************************************
AvNumStudyRangeStudy::AvNumStudyRangeStudy() {

  // define arrays of global variable names and types
  NumGVs = 4;
  NumExps = 3;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("ACCMINTOASSEMBLE");
  GVTypes[0]=strdup("short");
  GVNames[1]=strdup("ACCSIZE");
  GVTypes[1]=strdup("short");
  GVNames[2]=strdup("BINSIZE");
  GVTypes[2]=strdup("short");
  GVNames[3]=strdup("RecoverRate");
  GVTypes[3]=strdup("double");

  // create the arrays to store the values of each gv
  ACCMINTOASSEMBLEValues = new short[NumExps];
  ACCSIZEValues = new short[NumExps];
  BINSIZEValues = new short[NumExps];
  RecoverRateValues = new double[NumExps];

  // call methods to assign values to each gv
  SetValues_ACCMINTOASSEMBLE();
  SetValues_ACCSIZE();
  SetValues_BINSIZE();
  SetValues_RecoverRate();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//AvNumStudyRangeStudy Destructor
//******************************************************
AvNumStudyRangeStudy::~AvNumStudyRangeStudy() {
  delete [] ACCMINTOASSEMBLEValues;
  delete [] ACCSIZEValues;
  delete [] BINSIZEValues;
  delete [] RecoverRateValues;
  delete ThePVModel;
}


//******************************************************
// set values for ACCMINTOASSEMBLE
//******************************************************
void AvNumStudyRangeStudy::SetValues_ACCMINTOASSEMBLE() {
  ACCMINTOASSEMBLEValues[0] = 6;
  ACCMINTOASSEMBLEValues[1] = 6;
  ACCMINTOASSEMBLEValues[2] = 6;
}


//******************************************************
// set values for ACCSIZE
//******************************************************
void AvNumStudyRangeStudy::SetValues_ACCSIZE() {
  ACCSIZEValues[0] = 12;
  ACCSIZEValues[1] = 12;
  ACCSIZEValues[2] = 12;
}


//******************************************************
// set values for BINSIZE
//******************************************************
void AvNumStudyRangeStudy::SetValues_BINSIZE() {
  BINSIZEValues[0] = 8;
  BINSIZEValues[1] = 8;
  BINSIZEValues[2] = 8;
}


//******************************************************
// set values for RecoverRate
//******************************************************
void AvNumStudyRangeStudy::SetValues_RecoverRate() {
  RecoverRateValues[0] = 0.006;
  RecoverRateValues[1] = 0.009;
  RecoverRateValues[2] = 0.012;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void AvNumStudyRangeStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "ACCMINTOASSEMBLE\tshort\t" << ACCMINTOASSEMBLE << endl;
  cout << "ACCSIZE\tshort\t" << ACCSIZE << endl;
  cout << "BINSIZE\tshort\t" << BINSIZE << endl;
  cout << "RecoverRate\tdouble\t" << RecoverRate << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *AvNumStudyRangeStudy::GetGVValue(char *TheGVName) {
  if (strcmp("ACCMINTOASSEMBLE", TheGVName) == 0)
    return &ACCMINTOASSEMBLE;
  else if (strcmp("ACCSIZE", TheGVName) == 0)
    return &ACCSIZE;
  else if (strcmp("BINSIZE", TheGVName) == 0)
    return &BINSIZE;
  else if (strcmp("RecoverRate", TheGVName) == 0)
    return &RecoverRate;
  else 
    cerr<<"!! AvNumStudyRangeStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void AvNumStudyRangeStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("ACCMINTOASSEMBLE", TheGVName) == 0)
    ACCMINTOASSEMBLE = *(short *)TheGVValue;
  else if (strcmp("ACCSIZE", TheGVName) == 0)
    ACCSIZE = *(short *)TheGVValue;
  else if (strcmp("BINSIZE", TheGVName) == 0)
    BINSIZE = *(short *)TheGVValue;
  else if (strcmp("RecoverRate", TheGVName) == 0)
    RecoverRate = *(double *)TheGVValue;
  else 
    cerr<<"!! AvNumStudyRangeStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void AvNumStudyRangeStudy::SetGVs(int expNum) {
  ACCMINTOASSEMBLE = ACCMINTOASSEMBLEValues[expNum];
  ACCSIZE = ACCSIZEValues[expNum];
  BINSIZE = BINSIZEValues[expNum];
  RecoverRate = RecoverRateValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new AvNumStudyRangeStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* AvNumStudyRangeStudy::GetPVModel(bool expandTimeArrays) {
  if (ThePVModel!=NULL)
    delete ThePVModel;
  // create the PV model
  ThePVModel=new AvNumInBinsPVModel(expandTimeArrays);
  return ThePVModel;
}


