#ifndef memory_moduleSAN_H_
#define memory_moduleSAN_H_

#include "Cpp/BaseClasses/EmptyGroup.h"
#include "Cpp/BaseClasses/PreselectGroup.h"
#include "Cpp/BaseClasses/PostselectGroup.h"
#include "Cpp/BaseClasses/state/StructStateVariable.h"
#include "Cpp/BaseClasses/state/ArrayStateVariable.h"
#include "Cpp/BaseClasses/SAN/SANModel.h" 
#include "Cpp/BaseClasses/SAN/Place.h"
#include "Cpp/BaseClasses/SAN/ExtendedPlace.h"
extern double failure_rate;
extern double RAM_cov;
extern double mem_cov;
extern double comp_cov;
extern short num_comp;
extern UserDistributions* TheDistribution;

void MemoryError();

#ifndef _mystruct_header_
#define _mystruct_header_
typedef struct mystruct_state {short a; int b; char c; };

class mystruct: public StructStateVariable<mystruct_state> {
  public:
  ExtendedPlace<short>* a;
  ExtendedPlace<int>* b;
  ExtendedPlace<char>* c;
  mystruct(char* name):
    StructStateVariable<mystruct_state>(name, "") {
    a = new ExtendedPlace<short>("a", name);
    fields.push_back(field(Short, a));
    b = new ExtendedPlace<int>("b", name);
    fields.push_back(field(Int, b));
    c = new ExtendedPlace<char>("c", name);
    fields.push_back(field(Char, c));
    }
  mystruct(char* name, char* fullname):
    StructStateVariable<mystruct_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    a = new ExtendedPlace<short>("a", varname);
    fields.push_back(field(Short, a));
    b = new ExtendedPlace<int>("b", varname);
    fields.push_back(field(Int, b));
    c = new ExtendedPlace<char>("c", varname);
    fields.push_back(field(Char, c));
    }
  mystruct(char* name, mystruct_state initialValue): 
    StructStateVariable<mystruct_state>(name, "") {
    a = new ExtendedPlace<short>("a", name, initialValue.a);
    fields.push_back(field(Short, a));
    b = new ExtendedPlace<int>("b", name, initialValue.b);
    fields.push_back(field(Int, b));
    c = new ExtendedPlace<char>("c", name, initialValue.c);
    fields.push_back(field(Char, c));
    }
  mystruct(char* name, char* fullname, mystruct_state initialValue): 
    StructStateVariable<mystruct_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    a = new ExtendedPlace<short>("a", varname, initialValue.a);
    fields.push_back(field(Short, a));
    sprintf(varname, "%s.%s", fullname, name);
    b = new ExtendedPlace<int>("b", varname, initialValue.b);
    fields.push_back(field(Int, b));
    sprintf(varname, "%s.%s", fullname, name);
    c = new ExtendedPlace<char>("c", varname, initialValue.c);
    fields.push_back(field(Char, c));
    }
  ~mystruct() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif
#ifndef _myarray_header_
#define _myarray_header_

typedef mystruct_state myarray_state[4];
class myarray: public ArrayStateVariable<mystruct > {
  public:
  myarray(char* name, char* fullname):
    ArrayStateVariable<mystruct >(name, fullname, Struct ,4) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new mystruct(varname, fqname));
    }
  }

  myarray(char* name):
    ArrayStateVariable<mystruct >(name, "", Struct ,4) {
    char varname[12];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new mystruct(varname, name));
    }
  }

  myarray(char* name, char* fullname, mystruct_state & initialValue):
    ArrayStateVariable<mystruct >(name, fullname, Struct ,4) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new mystruct(varname, fqname, initialValue));
    }
  }

  myarray(char* name, mystruct_state & initialValue):
    ArrayStateVariable<mystruct >(name, "", Struct ,4) {
    char varname[12];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new mystruct(varname, name, initialValue));
    }
  }

  myarray(char* name, char* fullname, mystruct_state * initialValue):
    ArrayStateVariable<mystruct >(name, fullname, Struct ,4) {
    char varname[12];
    char fqname[strlen(fullname) + strlen(name) + 2];
    sprintf(fqname, "%s%s", fullname, name);    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new mystruct(varname, fqname, initialValue[i]));
    }
  }

  myarray(char* name, mystruct_state * initialValue):
    ArrayStateVariable<mystruct >(name, "", Struct ,4) {
    char varname[12];
    for (int i=0;i<4;i++) {
      sprintf(varname,"%d",i);
      fields.push_back(new mystruct(varname, name, initialValue[i]));
    }
  }
  ~myarray() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i);
  }
};
#endif
#ifndef _yourstruct_header_
#define _yourstruct_header_
typedef struct yourstruct_state {mystruct_state ourstruct; };

class yourstruct: public StructStateVariable<yourstruct_state> {
  public:
  mystruct* ourstruct;
  yourstruct(char* name):
    StructStateVariable<yourstruct_state>(name, "") {
    ourstruct = new mystruct("ourstruct", name);
    fields.push_back(field(Struct, ourstruct));
    }
  yourstruct(char* name, char* fullname):
    StructStateVariable<yourstruct_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ourstruct = new mystruct("ourstruct", varname);
    fields.push_back(field(Struct, ourstruct));
    }
  yourstruct(char* name, yourstruct_state initialValue): 
    StructStateVariable<yourstruct_state>(name, "") {
    ourstruct = new mystruct("ourstruct", name, initialValue.ourstruct);
    fields.push_back(field(Struct, ourstruct));
    }
  yourstruct(char* name, char* fullname, yourstruct_state initialValue): 
    StructStateVariable<yourstruct_state>(name, fullname) {
    char varname[strlen(fullname) + strlen(name) + 2];
    sprintf(varname, "%s.%s", fullname, name);
    ourstruct = new mystruct("ourstruct", varname, initialValue.ourstruct);
    fields.push_back(field(Struct, ourstruct));
    }
  ~yourstruct() {
    for (fieldIterator i=fields.begin();i!=fields.end();++i)
      delete (*i).field;
  }
};
#endif

/*********************************************************************
               memory_moduleSAN Submodel Definition                   
*********************************************************************/

class memory_moduleSAN:public SANModel{
public:

class interface_chip_failureActivity_case1:public Activity {
public:

  Place* interface_chips;
  short* interface_chips_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_chips;
  short* memory_chips_Mobius_Mark;

  double* TheDistributionParameters;
  interface_chip_failureActivity_case1();
  ~interface_chip_failureActivity_case1();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // interface_chip_failureActivity_case1Activity

class interface_chip_failureActivity_case2:public Activity {
public:

  Place* interface_chips;
  short* interface_chips_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_chips;
  short* memory_chips_Mobius_Mark;

  double* TheDistributionParameters;
  interface_chip_failureActivity_case2();
  ~interface_chip_failureActivity_case2();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // interface_chip_failureActivity_case2Activity

class interface_chip_failureActivity_case3:public Activity {
public:

  Place* interface_chips;
  short* interface_chips_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_chips;
  short* memory_chips_Mobius_Mark;

  double* TheDistributionParameters;
  interface_chip_failureActivity_case3();
  ~interface_chip_failureActivity_case3();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // interface_chip_failureActivity_case3Activity

class memory_chip_failureActivity_case1:public Activity {
public:

  Place* memory_chips;
  short* memory_chips_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;

  double* TheDistributionParameters;
  memory_chip_failureActivity_case1();
  ~memory_chip_failureActivity_case1();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // memory_chip_failureActivity_case1Activity

class memory_chip_failureActivity_case2:public Activity {
public:

  Place* memory_chips;
  short* memory_chips_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;
  Place* interface_chips;
  short* interface_chips_Mobius_Mark;

  double* TheDistributionParameters;
  memory_chip_failureActivity_case2();
  ~memory_chip_failureActivity_case2();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // memory_chip_failureActivity_case2Activity

class memory_chip_failureActivity_case3:public Activity {
public:

  Place* memory_chips;
  short* memory_chips_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;
  Place* interface_chips;
  short* interface_chips_Mobius_Mark;

  double* TheDistributionParameters;
  memory_chip_failureActivity_case3();
  ~memory_chip_failureActivity_case3();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // memory_chip_failureActivity_case3Activity

class memory_chip_failureActivity_case4:public Activity {
public:

  Place* memory_chips;
  short* memory_chips_Mobius_Mark;
  Place* computer_failed;
  short* computer_failed_Mobius_Mark;
  Place* memory_failed;
  short* memory_failed_Mobius_Mark;
  Place* interface_chips;
  short* interface_chips_Mobius_Mark;

  double* TheDistributionParameters;
  memory_chip_failureActivity_case4();
  ~memory_chip_failureActivity_case4();
  bool Enabled();
  void LinkVariables();
  double Weight();
  bool ReactivationPredicate();
  bool ReactivationFunction();
  double SampleDistribution();
  double* ReturnDistributionParameters();
  int Rank();
  BaseActionClass* Fire();
  double Rate();
}; // memory_chip_failureActivity_case4Activity

  //List of user-specified place names
  Place* memory_failed;
  Place* computer_failed;
  Place* interface_chips;
  Place* memory_chips;

  // Create instances of all actvities
  interface_chip_failureActivity_case1 interface_chip_failure_case1;
  interface_chip_failureActivity_case2 interface_chip_failure_case2;
  interface_chip_failureActivity_case3 interface_chip_failure_case3;
  memory_chip_failureActivity_case1 memory_chip_failure_case1;
  memory_chip_failureActivity_case2 memory_chip_failure_case2;
  memory_chip_failureActivity_case3 memory_chip_failure_case3;
  memory_chip_failureActivity_case4 memory_chip_failure_case4;
  //Create instances of all groups 
  EmptyGroup ImmediateGroup;
  PostselectGroup interface_chip_failureGroup;  PostselectGroup memory_chip_failureGroup;
  memory_moduleSAN();
  ~memory_moduleSAN();
  void CustomInitialization();

  void assignPlacesToActivitiesInst();
  void assignPlacesToActivitiesTimed();
}; // end memory_moduleSAN

#endif // memory_moduleSAN_H_
