
#include "Study/vary_num_comp/vary_num_compRangeStudy.h"

//******************************************************
//Global Variables
//******************************************************
double CPU_cov;
double IO_cov;
double RAM_cov;
double comp_cov;
double failure_rate;
double mem_cov;
short num_comp;
short num_mem_mod;

//********************************************************
//vary_num_compRangeStudy Constructor
//********************************************************
vary_num_compRangeStudy::vary_num_compRangeStudy() {

  // define arrays of global variable names and types
  NumGVs = 8;
  NumExps = 3;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("CPU_cov");
  GVTypes[0]=strdup("double");
  GVNames[1]=strdup("IO_cov");
  GVTypes[1]=strdup("double");
  GVNames[2]=strdup("RAM_cov");
  GVTypes[2]=strdup("double");
  GVNames[3]=strdup("comp_cov");
  GVTypes[3]=strdup("double");
  GVNames[4]=strdup("failure_rate");
  GVTypes[4]=strdup("double");
  GVNames[5]=strdup("mem_cov");
  GVTypes[5]=strdup("double");
  GVNames[6]=strdup("num_comp");
  GVTypes[6]=strdup("short");
  GVNames[7]=strdup("num_mem_mod");
  GVTypes[7]=strdup("short");

  // create the arrays to store the values of each gv
  CPU_covValues = new double[NumExps];
  IO_covValues = new double[NumExps];
  RAM_covValues = new double[NumExps];
  comp_covValues = new double[NumExps];
  failure_rateValues = new double[NumExps];
  mem_covValues = new double[NumExps];
  num_compValues = new short[NumExps];
  num_mem_modValues = new short[NumExps];

  // call methods to assign values to each gv
  SetValues_CPU_cov();
  SetValues_IO_cov();
  SetValues_RAM_cov();
  SetValues_comp_cov();
  SetValues_failure_rate();
  SetValues_mem_cov();
  SetValues_num_comp();
  SetValues_num_mem_mod();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//vary_num_compRangeStudy Destructor
//******************************************************
vary_num_compRangeStudy::~vary_num_compRangeStudy() {
  delete [] CPU_covValues;
  delete [] IO_covValues;
  delete [] RAM_covValues;
  delete [] comp_covValues;
  delete [] failure_rateValues;
  delete [] mem_covValues;
  delete [] num_compValues;
  delete [] num_mem_modValues;
  delete ThePVModel;
}


//******************************************************
// set values for CPU_cov
//******************************************************
void vary_num_compRangeStudy::SetValues_CPU_cov() {
  CPU_covValues[0] = 0.995;
  CPU_covValues[1] = 0.995;
  CPU_covValues[2] = 0.995;
}


//******************************************************
// set values for IO_cov
//******************************************************
void vary_num_compRangeStudy::SetValues_IO_cov() {
  IO_covValues[0] = 0.99;
  IO_covValues[1] = 0.99;
  IO_covValues[2] = 0.99;
}


//******************************************************
// set values for RAM_cov
//******************************************************
void vary_num_compRangeStudy::SetValues_RAM_cov() {
  RAM_covValues[0] = 0.998;
  RAM_covValues[1] = 0.998;
  RAM_covValues[2] = 0.998;
}


//******************************************************
// set values for comp_cov
//******************************************************
void vary_num_compRangeStudy::SetValues_comp_cov() {
  comp_covValues[0] = 0.95;
  comp_covValues[1] = 0.95;
  comp_covValues[2] = 0.95;
}


//******************************************************
// set values for failure_rate
//******************************************************
void vary_num_compRangeStudy::SetValues_failure_rate() {
  failure_rateValues[0] = 0.0008766;
  failure_rateValues[1] = 0.0008766;
  failure_rateValues[2] = 0.0008766;
}


//******************************************************
// set values for mem_cov
//******************************************************
void vary_num_compRangeStudy::SetValues_mem_cov() {
  mem_covValues[0] = 0.95;
  mem_covValues[1] = 0.95;
  mem_covValues[2] = 0.95;
}


//******************************************************
// set values for num_comp
//******************************************************
void vary_num_compRangeStudy::SetValues_num_comp() {
  num_compValues[0] = 1;
  num_compValues[1] = 2;
  num_compValues[2] = 3;
}


//******************************************************
// set values for num_mem_mod
//******************************************************
void vary_num_compRangeStudy::SetValues_num_mem_mod() {
  num_mem_modValues[0] = 3;
  num_mem_modValues[1] = 3;
  num_mem_modValues[2] = 3;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void vary_num_compRangeStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "CPU_cov\tdouble\t" << CPU_cov << endl;
  cout << "IO_cov\tdouble\t" << IO_cov << endl;
  cout << "RAM_cov\tdouble\t" << RAM_cov << endl;
  cout << "comp_cov\tdouble\t" << comp_cov << endl;
  cout << "failure_rate\tdouble\t" << failure_rate << endl;
  cout << "mem_cov\tdouble\t" << mem_cov << endl;
  cout << "num_comp\tshort\t" << num_comp << endl;
  cout << "num_mem_mod\tshort\t" << num_mem_mod << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *vary_num_compRangeStudy::GetGVValue(char *TheGVName) {
  if (strcmp("CPU_cov", TheGVName) == 0)
    return &CPU_cov;
  else if (strcmp("IO_cov", TheGVName) == 0)
    return &IO_cov;
  else if (strcmp("RAM_cov", TheGVName) == 0)
    return &RAM_cov;
  else if (strcmp("comp_cov", TheGVName) == 0)
    return &comp_cov;
  else if (strcmp("failure_rate", TheGVName) == 0)
    return &failure_rate;
  else if (strcmp("mem_cov", TheGVName) == 0)
    return &mem_cov;
  else if (strcmp("num_comp", TheGVName) == 0)
    return &num_comp;
  else if (strcmp("num_mem_mod", TheGVName) == 0)
    return &num_mem_mod;
  else 
    cerr<<"!! vary_num_compRangeStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void vary_num_compRangeStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("CPU_cov", TheGVName) == 0)
    CPU_cov = *(double *)TheGVValue;
  else if (strcmp("IO_cov", TheGVName) == 0)
    IO_cov = *(double *)TheGVValue;
  else if (strcmp("RAM_cov", TheGVName) == 0)
    RAM_cov = *(double *)TheGVValue;
  else if (strcmp("comp_cov", TheGVName) == 0)
    comp_cov = *(double *)TheGVValue;
  else if (strcmp("failure_rate", TheGVName) == 0)
    failure_rate = *(double *)TheGVValue;
  else if (strcmp("mem_cov", TheGVName) == 0)
    mem_cov = *(double *)TheGVValue;
  else if (strcmp("num_comp", TheGVName) == 0)
    num_comp = *(short *)TheGVValue;
  else if (strcmp("num_mem_mod", TheGVName) == 0)
    num_mem_mod = *(short *)TheGVValue;
  else 
    cerr<<"!! vary_num_compRangeStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void vary_num_compRangeStudy::SetGVs(int expNum) {
  CPU_cov = CPU_covValues[expNum];
  IO_cov = IO_covValues[expNum];
  RAM_cov = RAM_covValues[expNum];
  comp_cov = comp_covValues[expNum];
  failure_rate = failure_rateValues[expNum];
  mem_cov = mem_covValues[expNum];
  num_comp = num_compValues[expNum];
  num_mem_mod = num_mem_modValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new vary_num_compRangeStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* vary_num_compRangeStudy::GetPVModel(bool expandTimeArrays) {
  if (ThePVModel!=NULL)
    delete ThePVModel;
  // create the PV model
  ThePVModel=new multi_proc_PVPVModel(expandTimeArrays);
  return ThePVModel;
}


