
#include "Study/availability/availabilityRangeStudy.h"

//******************************************************
//Global Variables
//******************************************************
double repair_rate;

//********************************************************
//availabilityRangeStudy Constructor
//********************************************************
availabilityRangeStudy::availabilityRangeStudy() {

  // define arrays of global variable names and types
  NumGVs = 1;
  NumExps = 4;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("repair_rate");
  GVTypes[0]=strdup("double");

  // create the arrays to store the values of each gv
  repair_rateValues = new double[NumExps];

  // call methods to assign values to each gv
  SetValues_repair_rate();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//availabilityRangeStudy Destructor
//******************************************************
availabilityRangeStudy::~availabilityRangeStudy() {
  delete [] repair_rateValues;
  delete ThePVModel;
}


//******************************************************
// set values for repair_rate
//******************************************************
void availabilityRangeStudy::SetValues_repair_rate() {
  repair_rateValues[0] = 1.0;
  repair_rateValues[1] = 2.0;
  repair_rateValues[2] = 3.0;
  repair_rateValues[3] = 4.0;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void availabilityRangeStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "repair_rate\tdouble\t" << repair_rate << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *availabilityRangeStudy::GetGVValue(char *TheGVName) {
  if (strcmp("repair_rate", TheGVName) == 0)
    return &repair_rate;
  else 
    cerr<<"!! availabilityRangeStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void availabilityRangeStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("repair_rate", TheGVName) == 0)
    repair_rate = *(double *)TheGVValue;
  else 
    cerr<<"!! availabilityRangeStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void availabilityRangeStudy::SetGVs(int expNum) {
  repair_rate = repair_rateValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new availabilityRangeStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* availabilityRangeStudy::GetPVModel(bool expandTimeArrays) {
  if (ThePVModel!=NULL)
    delete ThePVModel;
  // create the PV model
  ThePVModel=new database_a_PVPVModel(expandTimeArrays);
  return ThePVModel;
}


