/*****************************************************************************\
 *                            RESTRICTED RIGHTS LEGEND
 *
 *      Use, duplication, or disclosure by the Government is subject to
 *      restrictions as set forth in subparagraph (c) (1) (ii) of the Rights in
 *      Technical Data and Computer Software Clause at DFARS 252.227-7013.
 *
 *      The Board of Trustees of the University of Illinois
 *      109 Coble Hall
 *      801 South Wright Street
 *      Champaign, IL 61820-6242
 *
 *      File:  PEPA.lex
 *
 *      Authors:  Graham Clark
 *      Creation Date: 2000
 *
 *      Copyright (c) 2000 The Board of Trustees of the University of Illinois
 *                         All rights reserved.
 *
\*****************************************************************************/


package Mobius.AtomicModels.PEPA;

import java.lang.System;
import java_cup.runtime.Symbol;
import edu.illinois.mobius.legacy.Mobius.AtomicModels.PEPA.sym;


class Utility {
  public static void assert
    (
     boolean expr
     )
      { 
	if (false == expr) {
	  throw (new Error("Error: Assertion failed."));
	}
      }
  
  private static final String errorMsg[] = {
    "Error: Unmatched end-of-comment punctuation.",
    "Error: Unmatched start-of-comment punctuation.",
    "Error: Unclosed string.",
    "Error: Illegal character."
    };
  
  public static final int E_ENDCOMMENT = 0; 
  public static final int E_STARTCOMMENT = 1; 
  public static final int E_UNCLOSEDSTR = 2; 
  public static final int E_UNMATCHED = 3; 

  public static void error
    (
     int code
     )
      {
	PEPADebug.println(errorMsg[code]);
      }
}

// class Yytoken {
//   Yytoken 
//     (
//      PepaToken token,
//      String text,
//      int line,
//      int charBegin,
//      int charEnd
//      )
//       {
// 	m_token = token;
// 	m_text = new String(text);
// 	m_line = line;
// 	m_charBegin = charBegin;
// 	m_charEnd = charEnd;
//       }

//   public PepaToken m_token;
//   public String m_text;
//   public int m_line;
//   public int m_charBegin;
//   public int m_charEnd;
// }
//%type Mobius.AtomicModels.PEPA.Symbol

%%

%cup
%{
  private int comment_count = 0;
  private int tabCount = 0;
%} 
%line
%char
%state COMMENT
%state CPPCOMMENT

ALPHA=[A-Za-z]
DIGIT=[0-9]
NONNEWLINETAB_WHITE_SPACE_CHAR=[\ \b]
WHITE_SPACE_CHAR=[\n\ \t\b\012]
STRING_TEXT=(\\\"|[^\n\"]|\\{WHITE_SPACE_CHAR}+\\)*
COMMENT_TEXT=([^/*\n]|[^*\n]"/"[^*\n]|[^/\n]"*"[^/\n]|"*"[^/\n]|"/"[^*\n])*
CPPCOMMENT_TEXT=([^\n])*

NUMBER=[0-9]+(\.[0-9]*)
NEWNUMBER=[0-9]
INTNUMBER=[0-9]+

%% 

<YYINITIAL> "," { return (new Symbol(sym.COMMA, yyline, tabCount)); }
<YYINITIAL> "." { PEPADebug.println("(Found a '.')"); return (new Symbol(sym.DOT, yyline, tabCount)); }
<YYINITIAL> "infty" { return (new Symbol(sym.TOP, yyline, tabCount, yytext())); }
<YYINITIAL> "Exp" { return (new Symbol(sym.EXP, yyline, tabCount)); }
<YYINITIAL> "Norm" { return (new Symbol(sym.NORM, yyline, tabCount)); }
<YYINITIAL> "Uni" { return (new Symbol(sym.UNIF, yyline, tabCount)); }
<YYINITIAL> "T" { return (new Symbol(sym.TOP, yyline, tabCount, yytext())); }
<YYINITIAL> "(" {         PEPADebug.println("(Found a '(')");
return (new Symbol(sym.LPAREN, yyline, tabCount)); }
<YYINITIAL> ")" { PEPADebug.println("(Found a ')')"); return (new Symbol(sym.RPAREN, yyline, tabCount)); }
<YYINITIAL> ";" { PEPADebug.println("(Found a ';')");
return (new Symbol(sym.SEMI, yyline, tabCount)); }
<YYINITIAL> "==" { return (new Symbol(sym.EQUALSEQUALS, yyline, tabCount)); }
<YYINITIAL> "=" { return (new Symbol(sym.EQUALS, yyline, tabCount)); }
<YYINITIAL> "!=" { return (new Symbol(sym.NOTEQUALS, yyline, tabCount)); }
<YYINITIAL> "!" { return (new Symbol(sym.ENOT, yyline, tabCount)); }
<YYINITIAL> "?" { return (new Symbol(sym.QUEST, yyline, tabCount)); }
<YYINITIAL> "=>" { return (new Symbol(sym.IMPLIES, yyline, tabCount)); }
<YYINITIAL> ":=" { return (new Symbol(sym.ASSIGNS, yyline, tabCount)); }
<YYINITIAL> "||" { return (new Symbol(sym.PAR, yyline, tabCount)); }
<YYINITIAL> "&&" { return (new Symbol(sym.EAND, yyline, tabCount)); }
<YYINITIAL> "<" { return (new Symbol(sym.LCOOP, yyline, tabCount)); }
<YYINITIAL> ">" { return (new Symbol(sym.RCOOP, yyline, tabCount)); }
<YYINITIAL> "[" { return (new Symbol(sym.LBRACK, yyline, tabCount)); }
<YYINITIAL> "]" { return (new Symbol(sym.RBRACK, yyline, tabCount)); }
<YYINITIAL> "{" { return (new Symbol(sym.LSET, yyline, tabCount)); }
<YYINITIAL> "}" { return (new Symbol(sym.RSET, yyline, tabCount)); }
<YYINITIAL> "*" { return (new Symbol(sym.MULT, yyline, tabCount)); }
<YYINITIAL> "-" { return (new Symbol(sym.MINUS, yyline, tabCount)); }
<YYINITIAL> "+" { return (new Symbol(sym.SUM, yyline, tabCount)); }
<YYINITIAL> "/" { return (new Symbol(sym.HIDE, yyline, tabCount)); }
<YYINITIAL> "#" { PEPADebug.println("(Found a '#')");
return (new Symbol(sym.HASH, yyline, tabCount)); }
<YYINITIAL> "%" { PEPADebug.println("(Found a '%')");
return (new Symbol(sym.PERC, yyline, tabCount)); }

<YYINITIAL> {NONNEWLINETAB_WHITE_SPACE_CHAR}+ { }

<YYINITIAL,COMMENT,CPPCOMMENT> \n { tabCount = 0;
                                    PEPADebug.println("reset tabcount"); }

<YYINITIAL> \t { tabCount++;
                 PEPADebug.println("found a tab, count now at "+tabCount); }

<YYINITIAL> {NUMBER} {
        PEPADebug.println("(Found a number)" + yytext());
//	return (new Symbol(sym.NUMB,new Integer(yytext())));
	return (new Symbol(sym.NUMB, yyline, tabCount, new Double(yytext())));
}

<YYINITIAL> {INTNUMBER} {
        PEPADebug.println("(Found an int)" + yytext());
	return (new Symbol(sym.INTNUMB, yyline, tabCount, new Integer(yytext())));
}

<YYINITIAL> {ALPHA}({ALPHA}|{DIGIT}|[_\'])* {
        PEPADebug.println("(Found a string)"+yytext());
	return (new Symbol(sym.STRN, yyline, tabCount, yytext()));
}	

<YYINITIAL> "//" { yybegin(CPPCOMMENT); }
<YYINITIAL> "/*" { yybegin(COMMENT); comment_count = comment_count + 1; }

<CPPCOMMENT> {CPPCOMMENT_TEXT} { yybegin(YYINITIAL); }

<COMMENT> "/*" { comment_count = comment_count + 1; }
<COMMENT> "*/" { 
	comment_count = comment_count - 1; 
	Utility.assert(comment_count >= 0);
	if (comment_count == 0) {
    		yybegin(YYINITIAL);
	}
}
<COMMENT> {COMMENT_TEXT} { }

<YYINITIAL,COMMENT,CPPCOMMENT> . {
        PEPADebug.println("Illegal character: <" + yytext() + ">");
	Utility.error(Utility.E_UNMATCHED);
}







