
#include "Study/vary_access/vary_accessSetStudy.h"

//******************************************************
//Global Variables
//******************************************************
short accessrate;
double arrivalrate;

//********************************************************
//vary_accessSetStudy Constructor
//********************************************************
vary_accessSetStudy::vary_accessSetStudy() {

  // define arrays of global variable names and types
  NumGVs = 2;
  NumExps = 3;

  GVNames = new char*[NumGVs];
  GVTypes = new char*[NumGVs];
  GVNames[0]=strdup("accessrate");
  GVTypes[0]=strdup("short");
  GVNames[1]=strdup("arrivalrate");
  GVTypes[1]=strdup("double");

  // create the arrays to store the values of each gv
  accessrateValues = new short[NumExps];
  arrivalrateValues = new double[NumExps];

  // call methods to assign values to each gv
  SetValues_accessrate();
  SetValues_arrivalrate();
  SetDefaultMobiusRoot(MOBIUSROOT);
}


//******************************************************
//vary_accessSetStudy Destructor
//******************************************************
vary_accessSetStudy::~vary_accessSetStudy() {
  delete [] accessrateValues;
  delete [] arrivalrateValues;
  delete ThePVModel;
}


//******************************************************
// set values for accessrate
//******************************************************
void vary_accessSetStudy::SetValues_accessrate() {
  accessrateValues[0] = 8;
  accessrateValues[1] = 10;
  accessrateValues[2] = 12;
}


//******************************************************
// set values for arrivalrate
//******************************************************
void vary_accessSetStudy::SetValues_arrivalrate() {
  arrivalrateValues[0] = 0.02;
  arrivalrateValues[1] = 0.04;
  arrivalrateValues[2] = 0.05;
}



//******************************************************
//print values of gv (for debugging)
//******************************************************
void vary_accessSetStudy::PrintGlobalValues(int expNum) {
  if (NumGVs == 0) {
    cout<<"There are no global variables."<<endl;
    return;
  }

  SetGVs(expNum);

  cout<<"The Global Variable values for experiment "<<
    GetExpName(expNum)<<" are:"<<endl;
  cout << "accessrate\tshort\t" << accessrate << endl;
  cout << "arrivalrate\tdouble\t" << arrivalrate << endl;
}


//******************************************************
//retrieve the value of a global variable
//******************************************************
void *vary_accessSetStudy::GetGVValue(char *TheGVName) {
  if (strcmp("accessrate", TheGVName) == 0)
    return &accessrate;
  else if (strcmp("arrivalrate", TheGVName) == 0)
    return &arrivalrate;
  else 
    cerr<<"!! vary_accessSetStudy::GetGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
  return NULL;
}


//******************************************************
//override the value of a global variable
//******************************************************
void vary_accessSetStudy::OverrideGVValue(char *TheGVName,void *TheGVValue) {
  if (strcmp("accessrate", TheGVName) == 0)
    accessrate = *(short *)TheGVValue;
  else if (strcmp("arrivalrate", TheGVName) == 0)
    arrivalrate = *(double *)TheGVValue;
  else 
    cerr<<"!! vary_accessSetStudy::OverrideGVValue: Global Variable "<<TheGVName<<" does not exist."<<endl;
}


//******************************************************
//set the value of all global variables to the given exp
//******************************************************
void vary_accessSetStudy::SetGVs(int expNum) {
  accessrate = accessrateValues[expNum];
  arrivalrate = arrivalrateValues[expNum];
}


//******************************************************
//static class method called by solvers to create study 
//(and thus create all of the model)
//******************************************************
BaseStudyClass* GlobalStudyPtr = NULL;
BaseStudyClass * GenerateStudy() {
  if (GlobalStudyPtr == NULL)
    GlobalStudyPtr = new vary_accessSetStudy();
  return GlobalStudyPtr;
}

void DestructStudy() {
  delete GlobalStudyPtr;
  GlobalStudyPtr = NULL;
}
//******************************************************
//get and create the PVModel
//******************************************************
PVModel* vary_accessSetStudy::GetPVModel(bool expandTimeArrays) {
  if (ThePVModel!=NULL)
    delete ThePVModel;
  // create the PV model
  ThePVModel=new lan_rewardPVModel(expandTimeArrays);
  return ThePVModel;
}


